DatPySci commited on
Commit
7b03500
·
verified ·
1 Parent(s): 6dfef9c

Upload folder using huggingface_hub

Browse files
Qwen3.4B-Math-R1-CoT-SFT/README.md CHANGED
@@ -1,22 +1,22 @@
1
  ---
2
  library_name: transformers
3
  license: other
4
- base_model: models/Qwen3-4B-Base
5
  tags:
6
  - llama-factory
7
  - full
8
  - generated_from_trainer
9
  model-index:
10
- - name: sft
11
  results: []
12
  ---
13
 
14
  <!-- This model card has been generated automatically according to the information the Trainer had access to. You
15
  should probably proofread and complete it, then remove this comment. -->
16
 
17
- # sft
18
 
19
- This model is a fine-tuned version of [models/Qwen3-4B-Base](https://huggingface.co/models/Qwen3-4B-Base) on the open_thoughts dataset.
20
 
21
  ## Model description
22
 
@@ -40,8 +40,10 @@ The following hyperparameters were used during training:
40
  - eval_batch_size: 8
41
  - seed: 42
42
  - distributed_type: multi-GPU
43
- - gradient_accumulation_steps: 32
 
44
  - total_train_batch_size: 64
 
45
  - optimizer: Use OptimizerNames.ADAMW_TORCH_FUSED with betas=(0.9,0.999) and epsilon=1e-08 and optimizer_args=No additional optimizer arguments
46
  - lr_scheduler_type: cosine
47
  - lr_scheduler_warmup_ratio: 0.05
 
1
  ---
2
  library_name: transformers
3
  license: other
4
+ base_model: models/SDAR-4B-Base
5
  tags:
6
  - llama-factory
7
  - full
8
  - generated_from_trainer
9
  model-index:
10
+ - name: sft_block_4
11
  results: []
12
  ---
13
 
14
  <!-- This model card has been generated automatically according to the information the Trainer had access to. You
15
  should probably proofread and complete it, then remove this comment. -->
16
 
17
+ # sft_block_4
18
 
19
+ This model is a fine-tuned version of [models/SDAR-4B-Base](https://huggingface.co/models/SDAR-4B-Base) on the open_thoughts dataset.
20
 
21
  ## Model description
22
 
 
40
  - eval_batch_size: 8
41
  - seed: 42
42
  - distributed_type: multi-GPU
43
+ - num_devices: 2
44
+ - gradient_accumulation_steps: 16
45
  - total_train_batch_size: 64
46
+ - total_eval_batch_size: 16
47
  - optimizer: Use OptimizerNames.ADAMW_TORCH_FUSED with betas=(0.9,0.999) and epsilon=1e-08 and optimizer_args=No additional optimizer arguments
48
  - lr_scheduler_type: cosine
49
  - lr_scheduler_warmup_ratio: 0.05
Qwen3.4B-Math-R1-CoT-SFT/added_tokens.json CHANGED
@@ -5,6 +5,7 @@
5
  "<think>": 151667,
6
  "<tool_call>": 151657,
7
  "<tool_response>": 151665,
 
8
  "<|box_end|>": 151649,
9
  "<|box_start|>": 151648,
10
  "<|endoftext|>": 151643,
 
5
  "<think>": 151667,
6
  "<tool_call>": 151657,
7
  "<tool_response>": 151665,
8
+ "<|MASK|>": 151669,
9
  "<|box_end|>": 151649,
10
  "<|box_start|>": 151648,
11
  "<|endoftext|>": 151643,
Qwen3.4B-Math-R1-CoT-SFT/all_results.json CHANGED
@@ -1,9 +1,9 @@
1
  {
2
- "effective_tokens_per_sec": 24290.389169769056,
3
- "epoch": 4.0,
4
- "total_flos": 4.3530238414695694e+19,
5
- "train_loss": 0.15123671168837785,
6
- "train_runtime": 82201.3053,
7
- "train_samples_per_second": 1.977,
8
- "train_steps_per_second": 0.031
9
  }
 
1
  {
2
+ "effective_tokens_per_sec": 16342.776477265266,
3
+ "epoch": 3.9988184324537217,
4
+ "total_flos": 4.3565600076592054e+19,
5
+ "train_loss": 1.895302517482642,
6
+ "train_runtime": 61070.1581,
7
+ "train_samples_per_second": 2.661,
8
+ "train_steps_per_second": 0.042
9
  }
Qwen3.4B-Math-R1-CoT-SFT/config.json CHANGED
@@ -1,30 +1,43 @@
1
  {
2
  "architectures": [
3
- "Qwen3ForCausalLM"
4
  ],
5
  "attention_bias": false,
6
  "attention_dropout": 0.0,
 
 
 
 
 
 
7
  "bos_token_id": 151643,
 
8
  "eos_token_id": 151643,
 
 
9
  "head_dim": 128,
10
  "hidden_act": "silu",
11
  "hidden_size": 2560,
12
  "initializer_range": 0.02,
13
  "intermediate_size": 9728,
 
14
  "max_position_embeddings": 32768,
15
  "max_window_layers": 36,
16
- "model_type": "qwen3",
 
17
  "num_attention_heads": 32,
18
  "num_hidden_layers": 36,
19
  "num_key_value_heads": 8,
20
  "rms_norm_eps": 1e-06,
21
  "rope_scaling": null,
22
  "rope_theta": 1000000,
 
23
  "sliding_window": null,
24
  "tie_word_embeddings": true,
25
  "torch_dtype": "bfloat16",
26
  "transformers_version": "4.52.4",
27
  "use_cache": false,
 
28
  "use_sliding_window": false,
29
  "vocab_size": 151936
30
  }
 
1
  {
2
  "architectures": [
3
+ "SDARForCausalLM"
4
  ],
5
  "attention_bias": false,
6
  "attention_dropout": 0.0,
7
+ "auto_map": {
8
+ "AutoConfig": "configuration_sdar.SDARConfig",
9
+ "AutoModel": "modeling_sdar.SDARForCausalLM",
10
+ "AutoModelForCausalLM": "modeling_sdar.SDARForCausalLM"
11
+ },
12
+ "block_size": 4,
13
  "bos_token_id": 151643,
14
+ "debug": false,
15
  "eos_token_id": 151643,
16
+ "ep_size": 1,
17
+ "fuse_cross_entropy": true,
18
  "head_dim": 128,
19
  "hidden_act": "silu",
20
  "hidden_size": 2560,
21
  "initializer_range": 0.02,
22
  "intermediate_size": 9728,
23
+ "mask_token_id": 151669,
24
  "max_position_embeddings": 32768,
25
  "max_window_layers": 36,
26
+ "micro_forward": false,
27
+ "model_type": "sdar",
28
  "num_attention_heads": 32,
29
  "num_hidden_layers": 36,
30
  "num_key_value_heads": 8,
31
  "rms_norm_eps": 1e-06,
32
  "rope_scaling": null,
33
  "rope_theta": 1000000,
34
+ "skip_checkpoint": false,
35
  "sliding_window": null,
36
  "tie_word_embeddings": true,
37
  "torch_dtype": "bfloat16",
38
  "transformers_version": "4.52.4",
39
  "use_cache": false,
40
+ "use_deepep": false,
41
  "use_sliding_window": false,
42
  "vocab_size": 151936
43
  }
Qwen3.4B-Math-R1-CoT-SFT/configuration_sdar.py ADDED
@@ -0,0 +1,212 @@
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
1
+ # coding=utf-8
2
+ # Copyright 2024 The Qwen team, Alibaba Group and the HuggingFace Inc. team. All rights reserved.
3
+ #
4
+ # Licensed under the Apache License, Version 2.0 (the "License");
5
+ # you may not use this file except in compliance with the License.
6
+ # You may obtain a copy of the License at
7
+ #
8
+ # http://www.apache.org/licenses/LICENSE-2.0
9
+ #
10
+ # Unless required by applicable law or agreed to in writing, software
11
+ # distributed under the License is distributed on an "AS IS" BASIS,
12
+ # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
13
+ # See the License for the specific language governing permissions and
14
+ # limitations under the License.
15
+ """SDAR model configuration"""
16
+
17
+ from transformers.configuration_utils import PretrainedConfig
18
+ from transformers.modeling_rope_utils import rope_config_validation
19
+ from transformers.utils import logging
20
+
21
+
22
+ logger = logging.get_logger(__name__)
23
+
24
+
25
+ class SDARConfig(PretrainedConfig):
26
+ r"""
27
+ This is the configuration class to store the configuration of a [`SDARModel`]. It is used to instantiate a
28
+ SDAR model according to the specified arguments, defining the model architecture. Instantiating a configuration
29
+ with the defaults will yield a similar configuration to that of
30
+ SDAR-1.7B [DiffuOpen/SDAR-1.7B-Chat](https://huggingface.co/DiffuOpen/SDAR-1.7B-Chat/).
31
+
32
+ Configuration objects inherit from [`PretrainedConfig`] and can be used to control the model outputs. Read the
33
+ documentation from [`PretrainedConfig`] for more information.
34
+
35
+
36
+ Args:
37
+ vocab_size (`int`, *optional*, defaults to 151936):
38
+ Vocabulary size of the SDAR model. Defines the number of different tokens that can be represented by the
39
+ `inputs_ids` passed when calling [`SDARModel`]
40
+ hidden_size (`int`, *optional*, defaults to 4096):
41
+ Dimension of the hidden representations.
42
+ intermediate_size (`int`, *optional*, defaults to 22016):
43
+ Dimension of the MLP representations.
44
+ num_hidden_layers (`int`, *optional*, defaults to 32):
45
+ Number of hidden layers in the Transformer encoder.
46
+ num_attention_heads (`int`, *optional*, defaults to 32):
47
+ Number of attention heads for each attention layer in the Transformer encoder.
48
+ num_key_value_heads (`int`, *optional*, defaults to 32):
49
+ This is the number of key_value heads that should be used to implement Grouped Query Attention. If
50
+ `num_key_value_heads=num_attention_heads`, the model will use Multi Head Attention (MHA), if
51
+ `num_key_value_heads=1` the model will use Multi Query Attention (MQA) otherwise GQA is used. When
52
+ converting a multi-head checkpoint to a GQA checkpoint, each group key and value head should be constructed
53
+ by meanpooling all the original heads within that group. For more details checkout [this
54
+ paper](https://arxiv.org/pdf/2305.13245.pdf). If it is not specified, will default to `32`.
55
+ head_dim (`int`, *optional*, defaults to 128):
56
+ The attention head dimension.
57
+ hidden_act (`str` or `function`, *optional*, defaults to `"silu"`):
58
+ The non-linear activation function (function or string) in the decoder.
59
+ max_position_embeddings (`int`, *optional*, defaults to 32768):
60
+ The maximum sequence length that this model might ever be used with.
61
+ initializer_range (`float`, *optional*, defaults to 0.02):
62
+ The standard deviation of the truncated_normal_initializer for initializing all weight matrices.
63
+ rms_norm_eps (`float`, *optional*, defaults to 1e-06):
64
+ The epsilon used by the rms normalization layers.
65
+ use_cache (`bool`, *optional*, defaults to `True`):
66
+ Whether or not the model should return the last key/values attentions (not used by all models). Only
67
+ relevant if `config.is_decoder=True`.
68
+ tie_word_embeddings (`bool`, *optional*, defaults to `False`):
69
+ Whether the model's input and output word embeddings should be tied.
70
+ rope_theta (`float`, *optional*, defaults to 10000.0):
71
+ The base period of the RoPE embeddings.
72
+ rope_scaling (`Dict`, *optional*):
73
+ Dictionary containing the scaling configuration for the RoPE embeddings. NOTE: if you apply new rope type
74
+ and you expect the model to work on longer `max_position_embeddings`, we recommend you to update this value
75
+ accordingly.
76
+ Expected contents:
77
+ `rope_type` (`str`):
78
+ The sub-variant of RoPE to use. Can be one of ['default', 'linear', 'dynamic', 'yarn', 'longrope',
79
+ 'llama3'], with 'default' being the original RoPE implementation.
80
+ `factor` (`float`, *optional*):
81
+ Used with all rope types except 'default'. The scaling factor to apply to the RoPE embeddings. In
82
+ most scaling types, a `factor` of x will enable the model to handle sequences of length x *
83
+ original maximum pre-trained length.
84
+ `original_max_position_embeddings` (`int`, *optional*):
85
+ Used with 'dynamic', 'longrope' and 'llama3'. The original max position embeddings used during
86
+ pretraining.
87
+ `attention_factor` (`float`, *optional*):
88
+ Used with 'yarn' and 'longrope'. The scaling factor to be applied on the attention
89
+ computation. If unspecified, it defaults to value recommended by the implementation, using the
90
+ `factor` field to infer the suggested value.
91
+ `beta_fast` (`float`, *optional*):
92
+ Only used with 'yarn'. Parameter to set the boundary for extrapolation (only) in the linear
93
+ ramp function. If unspecified, it defaults to 32.
94
+ `beta_slow` (`float`, *optional*):
95
+ Only used with 'yarn'. Parameter to set the boundary for interpolation (only) in the linear
96
+ ramp function. If unspecified, it defaults to 1.
97
+ `short_factor` (`List[float]`, *optional*):
98
+ Only used with 'longrope'. The scaling factor to be applied to short contexts (<
99
+ `original_max_position_embeddings`). Must be a list of numbers with the same length as the hidden
100
+ size divided by the number of attention heads divided by 2
101
+ `long_factor` (`List[float]`, *optional*):
102
+ Only used with 'longrope'. The scaling factor to be applied to long contexts (<
103
+ `original_max_position_embeddings`). Must be a list of numbers with the same length as the hidden
104
+ size divided by the number of attention heads divided by 2
105
+ `low_freq_factor` (`float`, *optional*):
106
+ Only used with 'llama3'. Scaling factor applied to low frequency components of the RoPE
107
+ `high_freq_factor` (`float`, *optional*):
108
+ Only used with 'llama3'. Scaling factor applied to high frequency components of the RoPE
109
+ attention_bias (`bool`, defaults to `False`, *optional*, defaults to `False`):
110
+ Whether to use a bias in the query, key, value and output projection layers during self-attention.
111
+ use_sliding_window (`bool`, *optional*, defaults to `False`):
112
+ Whether to use sliding window attention.
113
+ sliding_window (`int`, *optional*, defaults to 4096):
114
+ Sliding window attention (SWA) window size. If not specified, will default to `4096`.
115
+ max_window_layers (`int`, *optional*, defaults to 28):
116
+ The number of layers that use SWA (Sliding Window Attention). The bottom layers use SWA while the top use full attention.
117
+ attention_dropout (`float`, *optional*, defaults to 0.0):
118
+ The dropout ratio for the attention probabilities.
119
+
120
+ ```python
121
+ >>> from transformers import SDARModel, SDARConfig
122
+
123
+ >>> # Initializing a SDAR style configuration
124
+ >>> configuration = SDARConfig()
125
+
126
+ >>> # Initializing a model from the SDAR-8B style configuration
127
+ >>> model = SDARModel(configuration)
128
+
129
+ >>> # Accessing the model configuration
130
+ >>> configuration = model.config
131
+ ```"""
132
+
133
+ model_type = "sdar"
134
+ keys_to_ignore_at_inference = ["past_key_values"]
135
+
136
+ # Default tensor parallel plan for base model `SDAR`
137
+ base_model_tp_plan = {
138
+ "layers.*.self_attn.q_proj": "colwise",
139
+ "layers.*.self_attn.k_proj": "colwise",
140
+ "layers.*.self_attn.v_proj": "colwise",
141
+ "layers.*.self_attn.o_proj": "rowwise",
142
+ "layers.*.mlp.gate_proj": "colwise",
143
+ "layers.*.mlp.up_proj": "colwise",
144
+ "layers.*.mlp.down_proj": "rowwise",
145
+ }
146
+ base_model_pp_plan = {
147
+ "embed_tokens": (["input_ids"], ["inputs_embeds"]),
148
+ "layers": (["hidden_states", "attention_mask"], ["hidden_states"]),
149
+ "norm": (["hidden_states"], ["hidden_states"]),
150
+ }
151
+
152
+ def __init__(
153
+ self,
154
+ vocab_size=151936,
155
+ hidden_size=4096,
156
+ intermediate_size=22016,
157
+ num_hidden_layers=32,
158
+ num_attention_heads=32,
159
+ num_key_value_heads=32,
160
+ head_dim=128,
161
+ hidden_act="silu",
162
+ max_position_embeddings=32768,
163
+ initializer_range=0.02,
164
+ rms_norm_eps=1e-6,
165
+ use_cache=True,
166
+ tie_word_embeddings=False,
167
+ rope_theta=10000.0,
168
+ rope_scaling=None,
169
+ attention_bias=False,
170
+ use_sliding_window=False,
171
+ sliding_window=4096,
172
+ max_window_layers=28,
173
+ attention_dropout=0.0,
174
+ **kwargs,
175
+ ):
176
+ self.vocab_size = vocab_size
177
+ self.max_position_embeddings = max_position_embeddings
178
+ self.hidden_size = hidden_size
179
+ self.intermediate_size = intermediate_size
180
+ self.num_hidden_layers = num_hidden_layers
181
+ self.num_attention_heads = num_attention_heads
182
+ self.use_sliding_window = use_sliding_window
183
+ self.sliding_window = sliding_window # we check `use_sliding_window` in the modeling code
184
+ self.max_window_layers = max_window_layers
185
+
186
+ # for backward compatibility
187
+ if num_key_value_heads is None:
188
+ num_key_value_heads = num_attention_heads
189
+
190
+ self.num_key_value_heads = num_key_value_heads
191
+ self.head_dim = head_dim
192
+ self.hidden_act = hidden_act
193
+ self.initializer_range = initializer_range
194
+ self.rms_norm_eps = rms_norm_eps
195
+ self.use_cache = use_cache
196
+ self.rope_theta = rope_theta
197
+ self.rope_scaling = rope_scaling
198
+ self.attention_bias = attention_bias
199
+ self.attention_dropout = attention_dropout
200
+ # Validate the correctness of rotary position embeddings parameters
201
+ # BC: if there is a 'type' field, move it to 'rope_type'.
202
+ if self.rope_scaling is not None and "type" in self.rope_scaling:
203
+ self.rope_scaling["rope_type"] = self.rope_scaling["type"]
204
+ rope_config_validation(self)
205
+
206
+ super().__init__(
207
+ tie_word_embeddings=tie_word_embeddings,
208
+ **kwargs,
209
+ )
210
+
211
+
212
+ __all__ = ["SDARConfig"]
Qwen3.4B-Math-R1-CoT-SFT/fused_linear_diffusion_cross_entropy.py ADDED
@@ -0,0 +1,682 @@
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
1
+ # -*- coding: utf-8 -*-
2
+
3
+ # Code adapted from
4
+ # https://github.com/fla-org/flash-linear-attention/blob/main/fla/modules/fused_linear_cross_entropy.py
5
+ # Implementation of element-wise division of cross entropy loss
6
+
7
+
8
+ # Code adapted from
9
+ # https://github.com/linkedin/Liger-Kernel/blob/main/src/liger_kernel/ops/fused_linear_cross_entropy.py
10
+
11
+ from functools import partial
12
+ from typing import Optional, Tuple
13
+
14
+ import torch
15
+ import torch.nn as nn
16
+ import torch.nn.functional as F
17
+ import triton
18
+ import triton.language as tl
19
+ from torch.distributed import DeviceMesh
20
+ from torch.distributed.tensor import DTensor, Replicate, Shard, distribute_module
21
+ from torch.distributed.tensor.parallel import ParallelStyle
22
+
23
+ # The hard limit of TRITON_MAX_TENSOR_NUMEL is 1048576
24
+ # https://github.com/triton-lang/triton/blob/ba42a5c68fd0505f8c42f4202d53be0f8d9a5fe0/python/triton/language/core.py#L19
25
+ # However, setting limit as 65536 as in LayerNorm tutorial is faster because of less register spilling
26
+ # The optimal maximum block size depends on your hardware, your kernel, and your dtype
27
+ MAX_FUSED_SIZE = 65536 // 2
28
+
29
+
30
+ @triton.heuristics({
31
+ 'HAS_SCALE': lambda args: args['scale'] is not None
32
+ })
33
+ @triton.autotune(
34
+ configs=[
35
+ triton.Config({}, num_warps=num_warps)
36
+ for num_warps in [1, 2, 4, 8, 16, 32]
37
+ ],
38
+ key=['D']
39
+ )
40
+ @triton.jit
41
+ def logsumexp_fwd_kernel(
42
+ x,
43
+ z,
44
+ scale,
45
+ D: tl.constexpr,
46
+ B: tl.constexpr,
47
+ HAS_SCALE: tl.constexpr
48
+ ):
49
+ i_n, i_d = tl.program_id(0).to(tl.int64), tl.program_id(1).to(tl.int64)
50
+ o_d = i_d * B + tl.arange(0, B)
51
+ m_d = o_d < D
52
+
53
+ b_x = tl.load(x + i_n * D + o_d, mask=m_d, other=-float('inf'))
54
+ if HAS_SCALE:
55
+ b_x = b_x * scale
56
+ b_m = tl.max(b_x, 0)
57
+ b_z = tl.log(tl.sum(tl.exp(b_x - b_m), 0)) + b_m
58
+ tl.store(z + i_n * tl.cdiv(D, B) + i_d, b_z)
59
+
60
+
61
+ def logsumexp_fwd(
62
+ x,
63
+ scale: Optional[float] = None,
64
+ dtype: Optional[torch.dtype] = None
65
+ ):
66
+ r"""
67
+ Compute the logsumexp of the input tensor over the last dimension.
68
+
69
+ Args:
70
+ x (Tensor):
71
+ The input tensor of any shape.
72
+ scale (Optional[float]):
73
+ The scale applied to the input tensor. Default: `None`.
74
+ dtype (Optional[torch.dtype]):
75
+ The data type of the output tensor. Default: `None`.
76
+ Returns:
77
+ Tensor: The logsumexp of the input tensor.
78
+ """
79
+
80
+ shape = x.shape
81
+ x = x.view(-1, shape[-1])
82
+ N, D = x.shape
83
+ B = min(triton.next_power_of_2(D), 64 * 1024)
84
+ ND = triton.cdiv(D, B)
85
+
86
+ z = x.new_empty(N, ND, dtype=torch.float)
87
+ logsumexp_fwd_kernel[(N, ND)](
88
+ x=x,
89
+ z=z,
90
+ scale=scale,
91
+ D=D,
92
+ B=B
93
+ )
94
+ z = z.logsumexp(-1).view(*shape[:-1])
95
+ if dtype is not None and dtype != torch.float:
96
+ z = z.to(dtype)
97
+ return z
98
+
99
+ @triton.jit
100
+ def cross_entropy_kernel(
101
+ logits,
102
+ lse,
103
+ target,
104
+ p_mask,
105
+ loss,
106
+ total,
107
+ ignore_index,
108
+ label_smoothing: tl.constexpr,
109
+ logit_scale: tl.constexpr,
110
+ reduction: tl.constexpr,
111
+ V: tl.constexpr,
112
+ BV: tl.constexpr
113
+ ):
114
+ """
115
+ This kernel computes both cross entropy loss and the gradient of the input.
116
+ We only consider hard label + mean reduction for now.
117
+ Please refer to https://pytorch.org/docs/stable/generated/torch.nn.CrossEntropyLoss.html for the math.
118
+
119
+ Args:
120
+ logits:
121
+ Pointer to logits tensor.
122
+ lse:
123
+ Pointer to logsumexp tensor.
124
+ target: Pointer to target tensor.
125
+ loss:
126
+ Pointer to tensor to store the loss.
127
+ V (int):
128
+ The number of columns in the input tensor.
129
+ total (int):
130
+ The number of non-ignored classes.
131
+ ignore_index (int):
132
+ The index to ignore in the target.
133
+ label_smoothing (float):
134
+ The amount of smoothing when computing the loss, where 0.0 means no smoothing.
135
+ reduction (str):
136
+ The string for the reduction to apply
137
+ BV (int):
138
+ The block size for vocab.
139
+ """
140
+
141
+ # https://github.com/triton-lang/triton/issues/1058
142
+ # If B*T*V is too large, i_n * stride will overflow out of int32, so we convert to int64
143
+ i_n = tl.program_id(0).to(tl.int64)
144
+ NV = tl.cdiv(V, BV)
145
+
146
+ # 1. Load target first because if the target is ignore_index, we can return right away
147
+ b_y = tl.load(target + i_n)
148
+ # load p_mask
149
+ b_p_mask = tl.load(p_mask + i_n)
150
+
151
+ # 2. locate the start index
152
+ logits += i_n * V
153
+
154
+ if b_y == ignore_index:
155
+ # set all x as 0
156
+ for i in range(0, V, BV):
157
+ o_v = i + tl.arange(0, BV)
158
+ tl.store(logits + o_v, 0.0, mask=o_v < V)
159
+ return
160
+
161
+ # Online softmax: 2 loads + 1 store (compared with 3 loads + 1 store for the safe softmax)
162
+ # Refer to Algorithm 3 in the paper: https://arxiv.org/pdf/1805.02867
163
+
164
+ # 3. [Online softmax] first pass: compute logsumexp
165
+ # we did this in anouter kernel
166
+ b_l = tl.load(logits + b_y) * logit_scale
167
+ b_lse = tl.load(lse + i_n)
168
+
169
+ # 4. Calculate the loss
170
+ # loss = lse - logits_l
171
+ # celoss = -log(q_y) = -log(softmax(x_y))
172
+ b_loss = (b_lse - b_l) / b_p_mask # Diffusion Scaled '1/t'
173
+
174
+ # Label smoothing is a general case of normal cross entropy
175
+ # See the full derivation at https://github.com/linkedin/Liger-Kernel/pull/198#issue-2503665310
176
+ b_z = 0.0
177
+ eps = label_smoothing / V
178
+
179
+ # We need tl.debug_barrier() as mentioned in
180
+ # https://github.com/triton-lang/triton/blob/ba42a5c68fd0505f8c42f4202d53be0f8d9a5fe0/python/triton/ops/cross_entropy.py#L34
181
+ tl.debug_barrier()
182
+
183
+ # 5. [Online Softmax] Second pass: compute gradients
184
+ # For 'mean' reduction, gradients are normalized by number of non-ignored elements
185
+ # dx_y = (softmax(x_y) - 1) / N
186
+ # dx_i = softmax(x_i) / N, i != y
187
+ # For label smoothing:
188
+ # dx_i = (softmax(x_y) - label_smoothing / V) / N, i != y
189
+ # dx_y = (softmax(x_y) - label_smoothing / V - (1 - label_smoothing)) / N
190
+ # = dx_i - (1 - label_smoothing) / N
191
+ for iv in range(0, NV):
192
+ o_v = iv * BV + tl.arange(0, BV)
193
+ b_logits = tl.load(logits + o_v, mask=o_v < V, other=float('-inf')) * logit_scale
194
+ if label_smoothing > 0:
195
+ # scale X beforehand to avoid overflow
196
+ b_z += tl.sum(tl.where(o_v < V, -eps * b_logits, 0.0))
197
+ b_p = (tl.exp(b_logits - b_lse) - eps) * logit_scale
198
+ b_p /= b_p_mask # 修改
199
+ if reduction == "mean":
200
+ b_p = b_p / total
201
+ tl.store(logits + o_v, b_p, mask=o_v < V)
202
+
203
+ tl.debug_barrier()
204
+
205
+ # Orginal loss = H(q, p), with label smoothing regularization = H(q', p) and (label_smoothing / V) = eps
206
+ # H(q', p) = (1 - label_smoothing) * H(q, p) + label_smoothing * H(u, p)
207
+ # = (1 - label_smoothing) * H(q, p) + eps * sum(logsoftmax(x_i))
208
+ # By using m (global max of xi) and d (sum of e^(xi-m)), we can simplify as:
209
+ # = (1 - label_smoothing) * H(q, p) + (-sum(x_i * eps) + label_smoothing * (m + logd))
210
+ # Refer to H(q', p) in section 7 of the paper:
211
+ # https://arxiv.org/pdf/1512.00567
212
+ # pytorch:
213
+ # https://github.com/pytorch/pytorch/blob/2981534f54d49fa3a9755c9b0855e7929c2527f0/aten/src/ATen/native/LossNLL.cpp#L516
214
+ # See full derivation at https://github.com/linkedin/Liger-Kernel/pull/198#issuecomment-2333753087
215
+ if label_smoothing > 0:
216
+ b_loss = b_loss * (1 - label_smoothing) + (b_z + label_smoothing * b_lse)
217
+
218
+ # 6. Specially handle the i==y case where `dx_y = (softmax(x_y) - (1 - label_smoothing) / N`
219
+ b_l = tl.load(logits + b_y)
220
+
221
+ # Normalize the loss by the number of non-ignored elements if reduction is "mean"
222
+ if reduction == 'mean':
223
+ b_loss = b_loss / total
224
+ # b_l += (label_smoothing - 1) / total * logit_scale
225
+ # b_l has already been divided by b_p_mask and total
226
+ b_l += (label_smoothing - 1) / b_p_mask / total * logit_scale
227
+ else:
228
+ # b_l += (label_smoothing - 1) * logit_scale
229
+ b_l += (label_smoothing - 1) / b_p_mask * logit_scale
230
+
231
+ tl.store(loss + i_n, b_loss)
232
+ tl.store(logits + b_y, b_l)
233
+
234
+
235
+ @triton.jit
236
+ def elementwise_mul_kernel(
237
+ x,
238
+ g,
239
+ N: tl.constexpr,
240
+ B: tl.constexpr
241
+ ):
242
+ """
243
+ This function multiplies each element of the tensor pointed by x with the value pointed by g.
244
+ The multiplication is performed in-place on the tensor pointed by x.
245
+
246
+ Parameters:
247
+ x:
248
+ Pointer to the input tensor.
249
+ g:
250
+ Pointer to the gradient output value.
251
+ N (int):
252
+ The number of columns in the input tensor.
253
+ B (int):
254
+ The block size for Triton operations.
255
+ """
256
+
257
+ # Get the program ID and convert it to int64 to avoid overflow
258
+ i_x = tl.program_id(0).to(tl.int64)
259
+ o_x = i_x * B + tl.arange(0, B)
260
+
261
+ # Load the gradient output value
262
+ b_g = tl.load(g)
263
+ b_x = tl.load(x + o_x, mask=o_x < N)
264
+ tl.store(x + o_x, b_x * b_g, mask=o_x < N)
265
+
266
+
267
+ def fused_linear_cross_entropy_forward(
268
+ x: torch.Tensor,
269
+ target: torch.LongTensor,
270
+ weight: torch.Tensor,
271
+ bias: torch.Tensor = None,
272
+ p_mask: torch.Tensor = None,
273
+ ignore_index: int = -100,
274
+ label_smoothing: float = 0.0,
275
+ logit_scale: float = 1.0,
276
+ num_chunks: int = 8,
277
+ reduction: str = "mean"
278
+ ):
279
+ device = x.device
280
+ # inputs have shape: [N, H]
281
+ # materialized activations will have shape: [N, V]
282
+ # the increase in memory = [N, V]
283
+ # reduction can be achieved by partitioning the number of tokens N into smaller chunks.
284
+
285
+ # ideally, we would like to achieve the same memory consumption as [N, H],
286
+ # so the expected chunk size should be:
287
+ # NC = ceil(V / H)
288
+ # C = ceil(N / NC)
289
+ # for ex: N = 4096*4, V = 32000, H = 4096 ==> NC = 8, C = ceil(N / NC) = 2048
290
+ N, H, V = *x.shape, weight.shape[0]
291
+ BV = min(MAX_FUSED_SIZE, triton.next_power_of_2(V))
292
+ # TODO: in real cases, we may need to limit the number of chunks NC to
293
+ # ensure the precisions of accumulated gradients
294
+ NC = min(num_chunks, triton.cdiv(V, H))
295
+ C = triton.next_power_of_2(triton.cdiv(N, NC))
296
+ NC = triton.cdiv(N, C)
297
+
298
+ # [N, H]
299
+ dx = torch.zeros_like(x, device=device)
300
+ # [V, H]
301
+ dw = torch.zeros_like(weight, device=device, dtype=torch.float) if weight is not None else None
302
+ # [V]
303
+ db = torch.zeros_like(bias, device=device, dtype=torch.float) if bias is not None else None
304
+ # [N]
305
+ loss = torch.zeros(N, device=device, dtype=torch.float)
306
+
307
+ total = target.ne(ignore_index).sum().item()
308
+
309
+ for ic in range(NC):
310
+ start, end = ic * C, min((ic + 1) * C, N)
311
+ # [C, N]
312
+ c_x = x[start:end]
313
+ # when doing matmul, use the original precision
314
+ # [C, V]
315
+ c_logits = F.linear(c_x, weight, bias)
316
+ c_target = target[start:end]
317
+ c_p_mask = p_mask[start:end]
318
+ # [C]
319
+ # keep lse in fp32 to maintain precision
320
+ c_lse = logsumexp_fwd(c_logits, scale=logit_scale, dtype=torch.float)
321
+
322
+ # unreduced loss
323
+ c_loss = loss[start:end]
324
+
325
+ # Here we calculate the gradient of c_logits in place so we can save memory.
326
+ cross_entropy_kernel[(c_logits.shape[0],)](
327
+ logits=c_logits,
328
+ lse=c_lse,
329
+ target=c_target,
330
+ p_mask=c_p_mask,
331
+ loss=c_loss,
332
+ total=total,
333
+ ignore_index=ignore_index,
334
+ label_smoothing=label_smoothing,
335
+ logit_scale=logit_scale,
336
+ reduction=reduction,
337
+ V=V,
338
+ BV=BV,
339
+ num_warps=32
340
+ )
341
+
342
+ # gradient of logits is computed in-place by the above triton kernel and is of shape: C x V
343
+ # thus dx should be of shape: C x H
344
+ dx[start:end] = torch.mm(c_logits, weight)
345
+
346
+ # keep dw in fp32 to maintain precision
347
+ if weight is not None:
348
+ dw += c_logits.t() @ c_x
349
+
350
+ if bias is not None:
351
+ torch.add(input=db, other=c_logits.sum(0), out=db)
352
+
353
+ loss = loss.sum()
354
+ if dw is not None:
355
+ dw = dw.to(weight)
356
+ if db is not None:
357
+ db = db.to(bias)
358
+ return loss, dx, dw, db
359
+
360
+
361
+ def fused_linear_cross_entropy_backward(
362
+ do: torch.Tensor,
363
+ dx: torch.Tensor,
364
+ dw: torch.Tensor,
365
+ db: torch.Tensor
366
+ ):
367
+ # If cross entropy is the last layer, do is 1.0. Skip the mul to save time
368
+ if torch.ne(do, torch.tensor(1.0, device=do.device)):
369
+ # We use a Triton kernel instead of a PyTorch operation because modifying inputs in-place
370
+ # for gradient storage and backward multiple times causes anomalies with PyTorch but not with Triton.
371
+ N, H = dx.shape
372
+ B = min(MAX_FUSED_SIZE, triton.next_power_of_2(H))
373
+
374
+ elementwise_mul_kernel[(triton.cdiv(N * H, B),)](
375
+ x=dx,
376
+ g=do,
377
+ N=N*H,
378
+ B=B,
379
+ num_warps=32,
380
+ )
381
+
382
+ # handle dw
383
+ if dw is not None:
384
+ V, H = dw.shape
385
+ elementwise_mul_kernel[(triton.cdiv(V * H, B),)](
386
+ x=dw,
387
+ g=do,
388
+ N=V*H,
389
+ B=B,
390
+ num_warps=32,
391
+ )
392
+
393
+ if db is not None:
394
+ V = db.shape[0]
395
+ elementwise_mul_kernel[(triton.cdiv(V, B),)](
396
+ x=db,
397
+ g=do,
398
+ N=V,
399
+ B=B,
400
+ num_warps=32,
401
+ )
402
+ return dx, dw, db
403
+
404
+
405
+ class FusedLinearCrossEntropyFunction(torch.autograd.Function):
406
+
407
+ @staticmethod
408
+ def forward(
409
+ ctx,
410
+ x: torch.Tensor,
411
+ target: torch.LongTensor,
412
+ weight: torch.Tensor,
413
+ bias: torch.Tensor = None,
414
+ p_mask: torch.Tensor = None,
415
+ ignore_index: int = -100,
416
+ label_smoothing: float = 0.0,
417
+ logit_scale: float = 1.0,
418
+ num_chunks: int = 8,
419
+ reduction: str = "mean"
420
+ ):
421
+ """
422
+ Fusing the last linear layer with cross-entropy loss
423
+ Reference: https://github.com/mgmalek/efficient_cross_entropy
424
+
425
+ Handle the forward and backward pass of the final linear layer via cross-entropy loss by avoiding
426
+ the materialization of the large logits tensor. Since Cross Entropy Loss is the last layer, we can
427
+ compute the gradient at the forward pass. By doing so, we don't have to store the x and target
428
+ for the backward pass.
429
+
430
+ x (torch.Tensor): [batch_size * seq_len, hidden_size]
431
+ target (torch.LongTensor): [batch_size * seq_len]
432
+ where each value is in [0, vocab_size).
433
+ weight (torch.Tensor): [vocab_size, hidden_size]
434
+ where `vocab_size` is the number of classes.
435
+ bias (Optional[torch.Tensor]): [vocab_size]
436
+ where `vocab_size` is the number of classes.
437
+ p_mask(torch.Tensor): [batch_size * seq_len]
438
+ Its shape should be same as target.
439
+ ignore_index:
440
+ the index to ignore in the target.
441
+ label_smoothing:
442
+ the amount of smoothing when computing the loss, where 0.0 means no smoothing.
443
+ logit_scale: float = 1.0,
444
+ A scaling factor applied to the logits. Default: 1.0
445
+ num_chunks: int
446
+ The number of chunks to split the input tensor into for processing.
447
+ This can help optimize memory usage and computation speed.
448
+ Default: 8
449
+ reduction:
450
+ Specifies the reduction to apply to the output: 'mean' | 'sum'.
451
+ 'mean': the weighted mean of the output is taken,
452
+ 'sum': the output will be summed.
453
+ Default: 'mean'.
454
+ """
455
+ loss, dx, dw, db = fused_linear_cross_entropy_forward(
456
+ x,
457
+ target,
458
+ weight,
459
+ bias,
460
+ p_mask,
461
+ ignore_index,
462
+ label_smoothing,
463
+ logit_scale,
464
+ num_chunks,
465
+ reduction
466
+ )
467
+ # downcast to dtype and store for backward
468
+ ctx.save_for_backward(
469
+ dx.detach(),
470
+ dw.detach() if weight is not None else None,
471
+ db.detach() if bias is not None else None,
472
+ )
473
+ return loss
474
+
475
+ @staticmethod
476
+ def backward(ctx, do):
477
+ dx, dw, db = ctx.saved_tensors
478
+ dx, dw, db = fused_linear_cross_entropy_backward(do, dx, dw, db)
479
+ # 10 gradients should be returned, with `p_mask` having no grads
480
+ # Check the number of arguments in the `forward` method
481
+ return dx, None, dw, db, None, None, None, None, None, None
482
+
483
+
484
+ def fused_linear_cross_entropy_loss(
485
+ x: torch.Tensor,
486
+ target: torch.LongTensor,
487
+ weight: torch.Tensor,
488
+ bias: torch.Tensor = None,
489
+ p_mask: torch.Tensor = None,
490
+ ignore_index: int = -100,
491
+ label_smoothing: float = 0.0,
492
+ logit_scale: float = 1.0,
493
+ num_chunks: int = 8,
494
+ reduction: str = "mean"
495
+ ) -> Tuple[torch.Tensor, torch.Tensor]:
496
+ """
497
+ Args:
498
+ x (torch.Tensor): [batch_size * seq_len, hidden_size]
499
+ target (torch.LongTensor): [batch_size * seq_len]
500
+ where each value is in [0, vocab_size).
501
+ weight (torch.Tensor): [vocab_size, hidden_size]
502
+ where `vocab_size` is the number of classes.
503
+ bias (Optional[torch.Tensor]): [vocab_size]
504
+ where `vocab_size` is the number of classes.
505
+ p_mask(torch.Tensor): [batch_size * seq_len]
506
+ Its shape should be same as target.
507
+ ignore_index: int.
508
+ If target == ignore_index, the loss is set to 0.0.
509
+ label_smoothing: float
510
+ logit_scale: float
511
+ A scaling factor applied to the logits. Default: 1.0
512
+ num_chunks: int
513
+ The number of chunks to split the input tensor into for processing.
514
+ This can help optimize memory usage and computation speed.
515
+ Default: 8
516
+ reduction:
517
+ Specifies the reduction to apply to the output: 'mean' | 'sum'.
518
+ 'mean': the weighted mean of the output is taken,
519
+ 'sum': the output will be summed.
520
+ Default: 'mean'.
521
+ Returns:
522
+ losses: [batch,], float
523
+ """
524
+ return FusedLinearCrossEntropyFunction.apply(
525
+ x,
526
+ target,
527
+ weight,
528
+ bias,
529
+ p_mask,
530
+ ignore_index,
531
+ label_smoothing,
532
+ logit_scale,
533
+ num_chunks,
534
+ reduction
535
+ )
536
+
537
+
538
+ class FusedLinearDiffusionCrossEntropyLoss(nn.Module):
539
+
540
+ def __init__(
541
+ self,
542
+ ignore_index: int = -100,
543
+ label_smoothing: float = 0.0,
544
+ logit_scale: float = 1.0,
545
+ num_chunks: int = 8,
546
+ reduction: str = "mean"
547
+ ):
548
+ """
549
+ Args:
550
+ ignore_index: int.
551
+ If target == ignore_index, the loss is set to 0.0.
552
+ label_smoothing: float
553
+ logit_scale: float
554
+ A scaling factor applied to the logits. Default: 1.0
555
+ num_chunks: int
556
+ The number of chunks to split the input tensor into for processing.
557
+ This can help optimize memory usage and computation speed.
558
+ Default: 8
559
+ reduction:
560
+ Specifies the reduction to apply to the output: 'mean' | 'sum'.
561
+ 'mean': the weighted mean of the output is taken,
562
+ 'sum': the output will be summed.
563
+ Default: 'mean'.
564
+ """
565
+ super().__init__()
566
+
567
+ assert reduction in ["mean", "sum"], f"reduction: {reduction} is not supported"
568
+
569
+ self.ignore_index = ignore_index
570
+ self.label_smoothing = label_smoothing
571
+ self.logit_scale = logit_scale
572
+ self.num_chunks = num_chunks
573
+ self.reduction = reduction
574
+
575
+ @torch.compiler.disable
576
+ def forward(
577
+ self,
578
+ x: torch.Tensor,
579
+ target: torch.LongTensor,
580
+ weight: torch.Tensor,
581
+ bias: Optional[torch.Tensor] = None,
582
+ p_mask: torch.Tensor = None
583
+ ):
584
+ """
585
+ Args:
586
+ x (torch.Tensor): [batch_size, seq_len, hidden_size]
587
+ target (torch.LongTensor): [batch_size, seq_len]
588
+ where each value is in [0, V).
589
+ weight (torch.Tensor): [vocab_size, hidden_size]
590
+ where `vocab_size` is the number of classes.
591
+ bias (Optional[torch.Tensor]): [vocab_size]
592
+ where `vocab_size` is the number of classes.
593
+ p_mask(torch.Tensor): [batch_size, seq_len]
594
+ Its shape is same as target.
595
+ Shape: (1, packed_length) when varlen attn is used.
596
+ Returns:
597
+ loss
598
+
599
+ TODO:
600
+ follow https://github.com/ML-GSAI/LLaDA/blob/main/GUIDELINES.md#pre-training
601
+ ```py
602
+ unreduced_loss /= p_mask
603
+ ```
604
+ Scale the values of `unreduced_loss at different positions
605
+ """
606
+ if p_mask is None:
607
+ p_mask = torch.ones_like(target, dtype=torch.float, device=x.device)
608
+
609
+ x = x.contiguous().view(-1, x.shape[-1])
610
+ target = target.contiguous().view(-1)
611
+ weight = weight.contiguous()
612
+ bias = bias.contiguous() if bias else None
613
+ p_mask = p_mask.contiguous().view(-1)
614
+ l, d = x.shape
615
+ assert l == target.shape[0] == p_mask.shape[0], f"{x.shape=}, {target.shape=}, {p_mask.shape=}"
616
+
617
+ loss = fused_linear_cross_entropy_loss(
618
+ x,
619
+ target,
620
+ weight=weight,
621
+ bias=bias,
622
+ p_mask=p_mask,
623
+ ignore_index=self.ignore_index,
624
+ label_smoothing=self.label_smoothing,
625
+ logit_scale=self.logit_scale,
626
+ num_chunks=self.num_chunks,
627
+ reduction=self.reduction
628
+ )
629
+ return loss
630
+
631
+
632
+ class LinearLossParallel(ParallelStyle):
633
+ def __init__(
634
+ self,
635
+ *,
636
+ sequence_dim: int = 1,
637
+ use_local_output: bool = False,
638
+ ):
639
+ super().__init__()
640
+
641
+ self.sequence_sharding = (Shard(sequence_dim),)
642
+ self.use_local_output = use_local_output
643
+
644
+ @staticmethod
645
+ def _prepare_input_fn(sequence_sharding, mod, inputs, device_mesh):
646
+ x, target, weight, bias = inputs
647
+
648
+ if not isinstance(x, DTensor):
649
+ # assume the input passed in already sharded on the sequence dim and create the DTensor
650
+ x = DTensor.from_local(x, device_mesh, sequence_sharding)
651
+ if x.placements != sequence_sharding:
652
+ x = x.redistribute(placements=sequence_sharding, async_op=True)
653
+ if not isinstance(target, DTensor):
654
+ target = DTensor.from_local(target, device_mesh, [Replicate()])
655
+ if target.placements != sequence_sharding:
656
+ target = target.redistribute(placements=sequence_sharding, async_op=True)
657
+
658
+ if not isinstance(weight, DTensor):
659
+ weight = DTensor.from_local(weight, device_mesh, [Replicate()])
660
+ if weight.placements != [Replicate()]:
661
+ # we replicate the weight/bias in FLCE
662
+ weight = weight.redistribute(placements=[Replicate()], async_op=True)
663
+
664
+ if bias is not None and not isinstance(bias, DTensor):
665
+ bias = DTensor.from_local(bias, device_mesh, [Replicate()])
666
+ if bias is not None and bias.placements != [Replicate()]:
667
+ bias = bias.redistribute(placements=[Replicate()], async_op=True)
668
+
669
+ return x.to_local(), target.to_local(), weight.to_local(), bias.to_local() if bias is not None else bias
670
+
671
+ @staticmethod
672
+ def _prepare_output_fn(use_local_output, mod, outputs, device_mesh):
673
+ return outputs.to_local() if use_local_output else outputs
674
+
675
+ def _apply(self, module: nn.Module, device_mesh: DeviceMesh) -> nn.Module:
676
+ return distribute_module(
677
+ module,
678
+ device_mesh,
679
+ partition_fn=None,
680
+ input_fn=partial(self._prepare_input_fn, self.sequence_sharding),
681
+ output_fn=partial(self._prepare_output_fn, self.use_local_output)
682
+ )
Qwen3.4B-Math-R1-CoT-SFT/generation_config.json CHANGED
@@ -1,6 +1,7 @@
1
  {
 
2
  "bos_token_id": 151643,
3
  "eos_token_id": 151643,
4
- "max_new_tokens": 2048,
5
- "transformers_version": "4.52.4"
6
  }
 
1
  {
2
+ "_from_model_config": true,
3
  "bos_token_id": 151643,
4
  "eos_token_id": 151643,
5
+ "transformers_version": "4.52.4",
6
+ "use_cache": false
7
  }
Qwen3.4B-Math-R1-CoT-SFT/model-00001-of-00002.safetensors CHANGED
@@ -1,3 +1,3 @@
1
  version https://git-lfs.github.com/spec/v1
2
- oid sha256:4b79f2dc490d7dd405409615b2970b2913972c694d45d30c37c59e61e5fb427f
3
  size 4967215360
 
1
  version https://git-lfs.github.com/spec/v1
2
+ oid sha256:3cf721838f56081281667205fd4e2d29858a804aad837977f092eb179b428b35
3
  size 4967215360
Qwen3.4B-Math-R1-CoT-SFT/model-00002-of-00002.safetensors CHANGED
@@ -1,3 +1,3 @@
1
  version https://git-lfs.github.com/spec/v1
2
- oid sha256:dfbef910dba4b28ae32760f46c94c7ecc11cfdca910cee94c61fcb102a1ac409
3
  size 3855679144
 
1
  version https://git-lfs.github.com/spec/v1
2
+ oid sha256:309f36d853f5db682e8771eceee6fb90e8a963e9f64e0f8e8bb12178c7ce5baa
3
  size 3855679144
Qwen3.4B-Math-R1-CoT-SFT/modeling_sdar.py ADDED
@@ -0,0 +1,1233 @@
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
1
+ # This file is modified based on https://github.com/huggingface/transformers/blob/v4.52.4/src/transformers/models/qwen3/modeling_qwen3.py.
2
+ #
3
+ # 🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨
4
+ # This file was automatically generated from src/transformers/models/qwen3/modular_qwen3.py.
5
+ # Do NOT edit this file manually as any edits will be overwritten by the generation of
6
+ # the file from the modular. If any change should be done, please apply the change to the
7
+ # modular_qwen3.py file directly. One of our CI enforces this.
8
+ # 🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨🚨
9
+ # coding=utf-8
10
+ # Copyright 2025 The Qwen team, Alibaba Group and the HuggingFace Inc. team. All rights reserved.
11
+ #
12
+ # Licensed under the Apache License, Version 2.0 (the "License");
13
+ # you may not use this file except in compliance with the License.
14
+ # You may obtain a copy of the License at
15
+ #
16
+ # http://www.apache.org/licenses/LICENSE-2.0
17
+ #
18
+ # Unless required by applicable law or agreed to in writing, software
19
+ # distributed under the License is distributed on an "AS IS" BASIS,
20
+ # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
21
+ # See the License for the specific language governing permissions and
22
+ # limitations under the License.
23
+
24
+ from typing import Callable, Optional, Tuple, Union, List
25
+
26
+ import torch
27
+ from torch import nn
28
+ from einops import rearrange
29
+
30
+ from transformers.activations import ACT2FN
31
+ from transformers.cache_utils import Cache, DynamicCache, SlidingWindowCache, StaticCache
32
+ from transformers.generation import GenerationMixin
33
+ from transformers.integrations import use_kernel_forward_from_hub
34
+ from transformers.modeling_attn_mask_utils import AttentionMaskConverter
35
+ from transformers.modeling_flash_attention_utils import FlashAttentionKwargs
36
+ from transformers.modeling_layers import GradientCheckpointingLayer
37
+ from transformers.modeling_outputs import (
38
+ BaseModelOutputWithPast,
39
+ CausalLMOutputWithPast,
40
+ QuestionAnsweringModelOutput,
41
+ SequenceClassifierOutputWithPast,
42
+ TokenClassifierOutput,
43
+ )
44
+ from transformers.modeling_rope_utils import ROPE_INIT_FUNCTIONS, dynamic_rope_update
45
+ from transformers.modeling_utils import ALL_ATTENTION_FUNCTIONS, PreTrainedModel
46
+ from transformers.processing_utils import Unpack
47
+ from transformers.utils import LossKwargs, auto_docstring, can_return_tuple, is_torch_flex_attn_available, logging
48
+ from .configuration_sdar import SDARConfig
49
+ from .fused_linear_diffusion_cross_entropy import FusedLinearDiffusionCrossEntropyLoss
50
+
51
+ from flash_attn.ops.triton.layer_norm import rms_norm_fn as flash_rms_norm
52
+
53
+ import torch.nn.functional as F
54
+ try:
55
+ from flash_attn import flash_attn_func, flash_attn_varlen_func
56
+ from flash_attn.bert_padding import index_first_axis, pad_input, unpad_input
57
+ except:
58
+ pass
59
+
60
+ try:
61
+ from liger_kernel.ops.swiglu import LigerSiLUMulFunction # noqa: F401
62
+ liger_kernel_is_available = True
63
+ except ImportError:
64
+ liger_kernel_is_available = False
65
+
66
+
67
+ if is_torch_flex_attn_available():
68
+ from torch.nn.attention.flex_attention import BlockMask, create_block_mask, flex_attention
69
+ from transformers.integrations.flex_attention import make_flex_block_causal_mask
70
+
71
+
72
+ logger = logging.get_logger(__name__)
73
+
74
+
75
+ def modify_padded_position_ids_2d(position_ids: torch.LongTensor) -> torch.LongTensor:
76
+ """
77
+ 使用完全向量化的 PyTorch 操作修改一个 batch 的 packed position_ids。
78
+ 这个函数假设输入是一个 2D Tensor,形状为 (batch_size, sequence_length)。
79
+ 它会独立地处理 batch 中的每一行。
80
+
81
+ Args:
82
+ position_ids: 二维 PyTorch Tensor, shape (batch_size, sequence_length).
83
+
84
+ Returns:
85
+ 修改后的 position_ids Tensor, shape (batch_size, sequence_length).
86
+ """
87
+ if position_ids.dim() != 2:
88
+ raise ValueError(f"Input tensor must be 2D, but got {position_ids.dim()} dimensions.")
89
+
90
+ batch_size, seq_len = position_ids.shape
91
+ device = position_ids.device
92
+
93
+ col_indices = torch.arange(seq_len, device=device, dtype=position_ids.dtype).expand(batch_size, -1)
94
+ mask = (position_ids != 0)
95
+
96
+ masked_indices = col_indices * mask
97
+ last_nonzero_idx = torch.max(masked_indices, dim=1).values
98
+ has_nonzero = torch.any(mask, dim=1)
99
+ pad_start_idx = torch.where(has_nonzero, last_nonzero_idx + 1, torch.tensor(0, device=device, dtype=position_ids.dtype))
100
+
101
+ padding_mask = col_indices >= pad_start_idx.unsqueeze(1)
102
+ new_pad_values = col_indices - pad_start_idx.unsqueeze(1)
103
+ position_ids = torch.where(padding_mask, new_pad_values, position_ids)
104
+
105
+ return position_ids
106
+
107
+
108
+ def calculate_token_nums(position_ids: torch.Tensor):
109
+ """
110
+ 使用 PyTorch 高效计算一个批次中每个打包序列的长度。
111
+
112
+ Args:
113
+ position_ids (torch.Tensor): 一个 2D Tensor,形状为 (batch_size, sequence_length)。
114
+ 例如:tensor([[0,1,2,3,4,0,1,2,3,4,5,0,1,2,3,0,0,0]])
115
+ Returns:
116
+ list[list[int]]: 一个嵌套列表,包含每个批次项中各个序列的长度。
117
+ 例如:[[5, 6, 4, 1, 1, 1]]
118
+ """
119
+ # 检查输入是否为 2D Tensor
120
+ if position_ids.dim() != 2:
121
+ raise ValueError(f"输入必须是 2D Tensor,但得到了 {position_ids.dim()}D")
122
+
123
+ all_lengths = []
124
+
125
+ # 我们按批次逐行处理。因为每行的序列长度数量不同(ragged),
126
+ # 所以 Python 循环在批次维度上是最高效且最清晰的写法。
127
+ # 循环内部的操作是完全向量化的。
128
+ for pids_row in position_ids:
129
+ # 获取当前行的总长度
130
+ seq_len = pids_row.shape[0]
131
+
132
+ # 1. 找到所有值为 0 的元素的索引
133
+ # pids_row == 0 会返回一个布尔 Tensor: [True, False, ..., True, ...]
134
+ # torch.nonzero 会返回这些 True 值的索引
135
+ # .flatten() 将其从 (N, 1) 形状的 Tensor 变为 (N,) 形状
136
+ zero_indices = torch.nonzero(pids_row == 0).flatten()
137
+
138
+ # 2. 将序列的总长度作为一个额外的切分点添加到末尾
139
+ # 这对于计算最后一个序列的长度至关重要
140
+ # 注意:要确保新创建的 tensor 和原始 tensor 在同一个设备上 (cpu/cuda)
141
+ split_points = torch.cat([
142
+ zero_indices,
143
+ torch.tensor([seq_len], device=pids_row.device, dtype=zero_indices.dtype)
144
+ ])
145
+
146
+ # 3. 计算相邻切分点之间的差值,这就是我们想要的长度
147
+ # torch.diff([a, b, c, d]) 会返回 [b-a, c-b, d-c]
148
+ lengths = torch.diff(split_points)
149
+
150
+ all_lengths.append(lengths)
151
+
152
+ return all_lengths
153
+
154
+
155
+ def forward_add_noise_packed(
156
+ inputs_ids: torch.Tensor,
157
+ num_tokens_list: List[torch.Tensor],
158
+ prompt_mask: torch.Tensor,
159
+ mask_id: int,
160
+ eps: float = 1e-3,
161
+ max_tries: int = 10,
162
+ ) -> Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
163
+ """
164
+ 为一批打包(packed)序列的 token ID 添加噪声。
165
+
166
+ 此函数保留了为每个逻辑样本(在每个批次项内拼接)生成独立随机噪声率的逻辑。
167
+ 它会随机将一部分 token 的 ID 替换为 mask_id。
168
+ 这个过程会避开被 prompt_mask 标记的位置。
169
+
170
+ Args:
171
+ inputs_ids (torch.Tensor):
172
+ 输入的 token ID 张量,形状为 (bsz, total_tokens)。
173
+ num_tokens_list (List[torch.Tensor]):
174
+ 一个张量列表,长度为 bsz。列表中的每个张量记录了对应批次项中
175
+ 每个逻辑样本的长度。例如: [tensor([len1, len2]), tensor([len3, len4, len5])].
176
+ prompt_mask (torch.Tensor):
177
+ 布尔型张量,形状为 (bsz, total_tokens),值为 True 的位置表示是 prompt,
178
+ 不应添加噪声。
179
+ mask_id (int):
180
+ 用于替换的 mask token 的 ID。
181
+ eps (float):
182
+ 微小值,用于防止噪声率 t 恰好为 0,确保 p_mask > 0。
183
+ max_tries (int):
184
+ 为确保至少一个非 prompt token 被 mask,对每个批次项尝试的最大次数。
185
+
186
+ Returns:
187
+ Tuple[torch.Tensor, torch.Tensor, torch.Tensor]:
188
+ - noisy_input_ids (torch.Tensor):
189
+ 添加噪声后的 token ID 张量,形状为 (bsz, total_tokens)。
190
+ - final_masked_indices (torch.Tensor):
191
+ 布尔型张量,标记了哪些位置被实际 mask 了,形状为 (bsz, total_tokens)。
192
+ - p_masks (torch.Tensor):
193
+ 一个一维张量,包含了被 mask 的 token 对应的实际噪声率。
194
+ """
195
+ # 1. 验证和获取形状
196
+ bsz, total_tokens = inputs_ids.shape
197
+ device = inputs_ids.device
198
+
199
+ # 检查输入的一致性
200
+ assert len(num_tokens_list) == bsz, f"num_tokens_list 的长度 ({len(num_tokens_list)}) 必须等于 bsz ({bsz})"
201
+ assert prompt_mask.shape == (bsz, total_tokens), f"prompt_mask 形状不匹配, 期望 {(bsz, total_tokens)}, 得到 {prompt_mask.shape}"
202
+
203
+ # 准备结果容器
204
+ noisy_ids_list = []
205
+ final_masked_indices_list = []
206
+ p_masks_per_token_list = []
207
+
208
+ # 2. 在批次维度上迭代
209
+ # 这是处理不同打包结构最直接有效的方法
210
+ for i in range(bsz):
211
+ # 提取当前批次项的数据
212
+ current_ids = inputs_ids[i:i+1] # shape: (1, total_tokens)
213
+ current_num_tokens = num_tokens_list[i]
214
+ current_prompt_mask = prompt_mask[i:i+1] # shape: (1, total_tokens)
215
+
216
+ num_samples_in_item = len(current_num_tokens)
217
+ # 验证当前批次项的 token 总数是否匹配
218
+ assert total_tokens == torch.sum(current_num_tokens), \
219
+ f"批次项 {i} 的 num_tokens 之和 ({torch.sum(current_num_tokens)}) 与 total_tokens ({total_tokens}) 不匹配"
220
+
221
+ eligible_for_masking = ~current_prompt_mask
222
+
223
+ # 如果没有任何 token 可以被 mask,直接使用原始输入,并设置 p_mask 为 eps
224
+ if not eligible_for_masking.any():
225
+ noisy_ids_list.append(current_ids)
226
+ final_masked_indices_list.append(torch.zeros_like(current_prompt_mask, dtype=torch.bool))
227
+ # p_mask_per_token 的形状应为 (1, total_tokens) 以便后续拼接
228
+ p_masks_per_token_list.append(torch.full((1, total_tokens), eps, device=device, dtype=torch.float))
229
+ continue
230
+
231
+ # --- 尝试生成 mask,确保至少 mask 一个 token ---
232
+ final_masked_indices_item = torch.zeros_like(current_prompt_mask, dtype=torch.bool)
233
+ p_mask_per_token = None
234
+
235
+ for _ in range(max_tries):
236
+ # 为每个逻辑样本生成一个独立的噪声率 t
237
+ t = torch.rand(num_samples_in_item, device=device)
238
+ p_mask_per_sample = (1 - eps) * t + eps
239
+
240
+ # 将每个样本的噪声率扩展到其所有 token 上
241
+ p_mask_per_token_1d = torch.repeat_interleave(p_mask_per_sample, current_num_tokens)
242
+ p_mask_per_token = p_mask_per_token_1d.unsqueeze(0) # shape: (1, total_tokens)
243
+
244
+ # 根据噪声率生成随机 mask
245
+ masked_indices = torch.rand_like(p_mask_per_token) < p_mask_per_token
246
+ # 应用 prompt mask,确保 prompt 不被 mask
247
+ final_masked_indices_item = masked_indices & eligible_for_masking
248
+
249
+ # 如果成功 mask 了至少一个 token,则跳出尝试循环
250
+ if final_masked_indices_item.any():
251
+ break
252
+
253
+ # 如果 max_tries 之后仍然没有 mask 任何 token (极小概率),就强制 mask 一个可 mask 的 token
254
+ if not final_masked_indices_item.any():
255
+ eligible_indices = torch.nonzero(eligible_for_masking.squeeze(0), as_tuple=True)[0]
256
+ if len(eligible_indices) > 0:
257
+ # 随机选择一个可 mask 的位置
258
+ random_choice = torch.randint(0, len(eligible_indices), (1,)).item()
259
+ force_mask_idx = eligible_indices[random_choice]
260
+ final_masked_indices_item[0, force_mask_idx] = True
261
+
262
+
263
+ # --- 根据最终的 mask 生成带噪声的 IDs ---
264
+ noisy_ids_item = torch.where(
265
+ final_masked_indices_item,
266
+ mask_id,
267
+ current_ids
268
+ )
269
+
270
+ # 保存这个批次项的结果
271
+ noisy_ids_list.append(noisy_ids_item)
272
+ final_masked_indices_list.append(final_masked_indices_item)
273
+ p_masks_per_token_list.append(p_mask_per_token)
274
+
275
+ # 3. 将列表中的结果堆叠成最终的批处理张量
276
+ noisy_input_ids = torch.cat(noisy_ids_list, dim=0)
277
+ final_masked_indices = torch.cat(final_masked_indices_list, dim=0)
278
+ p_mask_full = torch.cat(p_masks_per_token_list, dim=0)
279
+
280
+ # 4. 提取被 mask 位置对应的噪声率
281
+ p_masks = p_mask_full[final_masked_indices]
282
+
283
+ return noisy_input_ids, final_masked_indices, p_masks
284
+
285
+
286
+ def block_diff_mask(b, h, q_idx, kv_idx, block_size=None, n=None):
287
+ """
288
+ Constructs the specialized block diffusion attention mask for training
289
+ composed of three masks:
290
+ - **Block Diagonal Mask (M_BD)**: Self-attention within noised blocks
291
+ - **Offset Block Causal Mask (M_OBC)**: Cross-attention for conditional context
292
+ - **Block Causal Mask (M_BC)**: Attention to update x0
293
+
294
+ Args:
295
+ b, h: Batch and head indices (ignored for mask logic).
296
+ q_idx, kv_idx: Query and Key indices.
297
+ seq_len: Total sequence length.
298
+ block_size: Defines the block structure.
299
+
300
+ Returns:
301
+ A boolean attention mask.
302
+ """
303
+
304
+ # Indicate whether token belongs to xt or x0
305
+ x0_flag_q = q_idx >= n
306
+ x0_flag_kv = kv_idx >= n
307
+
308
+ # Compute block indices
309
+ block_q = torch.where(
310
+ x0_flag_q == 1, (q_idx - n) // block_size, q_idx // block_size
311
+ )
312
+ block_kv = torch.where(
313
+ x0_flag_kv == 1, (kv_idx - n) // block_size, kv_idx // block_size
314
+ )
315
+
316
+ # **1. Block Diagonal Mask (M_BD) **
317
+ block_diagonal = (block_q == block_kv) & (x0_flag_q == x0_flag_kv)
318
+
319
+ # **2. Offset Block-Causal Mask (M_OBC) **
320
+ offset_block_causal = (block_q > block_kv) & (
321
+ x0_flag_kv == 1) & (x0_flag_q == 0)
322
+
323
+ # **3. Block-Causal Mask (M_BC) **
324
+ block_causal = (block_q >= block_kv) & (x0_flag_kv == 1) & (x0_flag_q == 1)
325
+
326
+ # **4. Combine Masks **
327
+ return block_diagonal | offset_block_causal | block_causal
328
+
329
+
330
+ def block_attn_mask(num_tokens, block_size, device):
331
+ masks = []
332
+ for i in range(len(num_tokens)):
333
+ cur_masks = []
334
+ for num in num_tokens[i]:
335
+ # 全部返回 n*n 而非 2n*2n
336
+ single_mask = block_diff_mask(
337
+ b=None,
338
+ h=None,
339
+ q_idx=torch.arange(num * 2, device=device)[:, None],
340
+ kv_idx=torch.arange(num * 2, device=device)[None, :],
341
+ block_size=block_size,
342
+ n=num,
343
+ )
344
+ cur_masks.append(single_mask)
345
+ masks.append(torch.block_diag(*cur_masks))
346
+ masks = torch.stack(masks, dim=0)
347
+ return masks
348
+
349
+
350
+ @torch.compile(fullgraph=True, mode="max-autotune-no-cudagraphs")
351
+ def fused_flex_attention(query, key, value, attention_mask, **kwargs):
352
+ return flex_attention(query, key, value, block_mask=attention_mask, **kwargs)
353
+
354
+
355
+ @use_kernel_forward_from_hub("RMSNorm")
356
+ class SDARRMSNorm(nn.Module):
357
+ def __init__(self, hidden_size, eps=1e-6):
358
+ """
359
+ SDARRMSNorm is equivalent to T5LayerNorm
360
+ """
361
+ super().__init__()
362
+ self.weight = nn.Parameter(torch.ones(hidden_size))
363
+ self.variance_epsilon = eps
364
+
365
+ def forward(self, hidden_states):
366
+ return flash_rms_norm(
367
+ hidden_states, weight=self.weight, bias=None, eps=self.variance_epsilon)
368
+ '''
369
+ input_dtype = hidden_states.dtype
370
+ hidden_states = hidden_states.to(torch.float32)
371
+ variance = hidden_states.pow(2).mean(-1, keepdim=True)
372
+ hidden_states = hidden_states * \
373
+ torch.rsqrt(variance + self.variance_epsilon)
374
+ return self.weight * hidden_states.to(input_dtype)
375
+ '''
376
+
377
+ def extra_repr(self):
378
+ return f"{tuple(self.weight.shape)}, eps={self.variance_epsilon}"
379
+
380
+
381
+ class SDARMLP(nn.Module):
382
+ def __init__(self, config):
383
+ super().__init__()
384
+ self.config = config
385
+ self.hidden_size = config.hidden_size
386
+ self.intermediate_size = config.intermediate_size
387
+ self.gate_proj = nn.Linear(
388
+ self.hidden_size, self.intermediate_size, bias=False)
389
+ self.up_proj = nn.Linear(
390
+ self.hidden_size, self.intermediate_size, bias=False)
391
+ self.down_proj = nn.Linear(
392
+ self.intermediate_size, self.hidden_size, bias=False)
393
+ self.act_fn = ACT2FN[config.hidden_act]
394
+
395
+ def forward(self, x):
396
+ if liger_kernel_is_available:
397
+ return self.down_proj(LigerSiLUMulFunction.apply(self.gate_proj(x), self.up_proj(x)))
398
+ else:
399
+ down_proj = self.down_proj(self.act_fn(
400
+ self.gate_proj(x)) * self.up_proj(x))
401
+ return down_proj
402
+
403
+
404
+ def rotate_half(x):
405
+ """Rotates half the hidden dims of the input."""
406
+ x1 = x[..., : x.shape[-1] // 2]
407
+ x2 = x[..., x.shape[-1] // 2:]
408
+ return torch.cat((-x2, x1), dim=-1)
409
+
410
+
411
+ def apply_rotary_pos_emb(q, k, cos, sin, position_ids=None, unsqueeze_dim=1):
412
+ """Applies Rotary Position Embedding to the query and key tensors.
413
+
414
+ Args:
415
+ q (`torch.Tensor`): The query tensor.
416
+ k (`torch.Tensor`): The key tensor.
417
+ cos (`torch.Tensor`): The cosine part of the rotary embedding.
418
+ sin (`torch.Tensor`): The sine part of the rotary embedding.
419
+ position_ids (`torch.Tensor`, *optional*):
420
+ Deprecated and unused.
421
+ unsqueeze_dim (`int`, *optional*, defaults to 1):
422
+ The 'unsqueeze_dim' argument specifies the dimension along which to unsqueeze cos[position_ids] and
423
+ sin[position_ids] so that they can be properly broadcasted to the dimensions of q and k. For example, note
424
+ that cos[position_ids] and sin[position_ids] have the shape [batch_size, seq_len, head_dim]. Then, if q and
425
+ k have the shape [batch_size, heads, seq_len, head_dim], then setting unsqueeze_dim=1 makes
426
+ cos[position_ids] and sin[position_ids] broadcastable to the shapes of q and k. Similarly, if q and k have
427
+ the shape [batch_size, seq_len, heads, head_dim], then set unsqueeze_dim=2.
428
+ Returns:
429
+ `tuple(torch.Tensor)` comprising of the query and key tensors rotated using the Rotary Position Embedding.
430
+ """
431
+ cos = cos.unsqueeze(unsqueeze_dim)
432
+ sin = sin.unsqueeze(unsqueeze_dim)
433
+ q_embed = (q * cos) + (rotate_half(q) * sin)
434
+ k_embed = (k * cos) + (rotate_half(k) * sin)
435
+ return q_embed, k_embed
436
+
437
+
438
+ def repeat_kv(hidden_states: torch.Tensor, n_rep: int) -> torch.Tensor:
439
+ """
440
+ This is the equivalent of torch.repeat_interleave(x, dim=1, repeats=n_rep). The hidden states go from (batch,
441
+ num_key_value_heads, seqlen, head_dim) to (batch, num_attention_heads, seqlen, head_dim)
442
+ """
443
+ batch, num_key_value_heads, slen, head_dim = hidden_states.shape
444
+ if n_rep == 1:
445
+ return hidden_states
446
+ hidden_states = hidden_states[:, :, None, :, :].expand(
447
+ batch, num_key_value_heads, n_rep, slen, head_dim)
448
+ return hidden_states.reshape(batch, num_key_value_heads * n_rep, slen, head_dim)
449
+
450
+
451
+ def eager_attention_forward(
452
+ module: nn.Module,
453
+ query: torch.Tensor,
454
+ key: torch.Tensor,
455
+ value: torch.Tensor,
456
+ attention_mask: Optional[torch.Tensor],
457
+ scaling: float,
458
+ dropout: float = 0.0,
459
+ **kwargs,
460
+ ):
461
+ key_states = repeat_kv(key, module.num_key_value_groups)
462
+ value_states = repeat_kv(value, module.num_key_value_groups)
463
+
464
+ attn_weights = torch.matmul(query, key_states.transpose(2, 3)) * scaling
465
+ if attention_mask is not None:
466
+ causal_mask = attention_mask[:, :, :, : key_states.shape[-2]]
467
+ attn_weights = attn_weights + causal_mask
468
+
469
+ attn_weights = nn.functional.softmax(
470
+ attn_weights, dim=-1, dtype=torch.float32).to(query.dtype)
471
+ attn_weights = nn.functional.dropout(
472
+ attn_weights, p=dropout, training=module.training)
473
+ attn_output = torch.matmul(attn_weights, value_states)
474
+ attn_output = attn_output.transpose(1, 2).contiguous()
475
+
476
+ return attn_output, attn_weights
477
+
478
+
479
+ class SDARAttention(nn.Module):
480
+ """Multi-headed attention from 'Attention Is All You Need' paper"""
481
+
482
+ def __init__(self, config: SDARConfig, layer_idx: int):
483
+ super().__init__()
484
+ self.config = config
485
+ self.layer_idx = layer_idx
486
+ self.head_dim = getattr(
487
+ config, "head_dim", config.hidden_size // config.num_attention_heads)
488
+ self.num_key_value_groups = config.num_attention_heads // config.num_key_value_heads
489
+ self.scaling = self.head_dim**-0.5
490
+ self.attention_dropout = config.attention_dropout
491
+ self.is_causal = True
492
+
493
+ self.hidden_size = config.hidden_size
494
+ self.num_attention_heads = config.num_attention_heads
495
+ self.num_key_value_heads = config.num_key_value_heads
496
+
497
+ self.q_proj = nn.Linear(
498
+ config.hidden_size, config.num_attention_heads * self.head_dim, bias=config.attention_bias
499
+ )
500
+ self.k_proj = nn.Linear(
501
+ config.hidden_size, config.num_key_value_heads * self.head_dim, bias=config.attention_bias
502
+ )
503
+ self.v_proj = nn.Linear(
504
+ config.hidden_size, config.num_key_value_heads * self.head_dim, bias=config.attention_bias
505
+ )
506
+ self.o_proj = nn.Linear(
507
+ config.num_attention_heads * self.head_dim, config.hidden_size, bias=config.attention_bias
508
+ )
509
+ # unlike olmo, only on the head dim!
510
+ self.q_norm = SDARRMSNorm(self.head_dim, eps=config.rms_norm_eps)
511
+ # thus post q_norm does not need reshape
512
+ self.k_norm = SDARRMSNorm(self.head_dim, eps=config.rms_norm_eps)
513
+ self.sliding_window = config.sliding_window
514
+ if not (
515
+ self.config.use_sliding_window
516
+ and getattr(self.config, "sliding_window", None) is not None
517
+ and self.layer_idx >= self.config.max_window_layers
518
+ ):
519
+ self.sliding_window = None
520
+
521
+ def forward(
522
+ self,
523
+ hidden_states: torch.Tensor,
524
+ position_embeddings: Tuple[torch.Tensor, torch.Tensor],
525
+ attention_mask: Optional[torch.Tensor],
526
+ past_key_value: Optional[Cache] = None,
527
+ cache_position: Optional[torch.LongTensor] = None,
528
+ **kwargs: Unpack[FlashAttentionKwargs],
529
+ ) -> Tuple[torch.Tensor, Optional[torch.Tensor], Optional[Tuple[torch.Tensor]]]:
530
+ input_shape = hidden_states.shape[:-1]
531
+ bsz, q_len = input_shape
532
+ hidden_shape = (*input_shape, -1, self.head_dim)
533
+
534
+ query_states = self.q_norm(self.q_proj(
535
+ hidden_states).view(hidden_shape)).transpose(1, 2)
536
+ key_states = self.k_norm(self.k_proj(
537
+ hidden_states).view(hidden_shape)).transpose(1, 2)
538
+ value_states = self.v_proj(hidden_states).view(
539
+ hidden_shape).transpose(1, 2)
540
+
541
+ cos, sin = position_embeddings
542
+ query_states, key_states = apply_rotary_pos_emb(
543
+ query_states, key_states, cos, sin)
544
+
545
+ if past_key_value is not None and kwargs.get("store_kv", False):
546
+ # sin and cos are specific to RoPE models; cache_position needed for the static cache
547
+ key_states, value_states = past_key_value.update(
548
+ key_states, value_states, self.layer_idx)
549
+ elif past_key_value is not None and not kwargs.get("store_kv", False) and len(past_key_value) > self.layer_idx:
550
+ # only retrive, do not store kv
551
+ past_key_states, past_value_states = past_key_value[self.layer_idx]
552
+ key_states = torch.cat(
553
+ [past_key_states, key_states], dim=-2)
554
+ value_states = torch.cat(
555
+ [past_value_states, value_states], dim=-2)
556
+
557
+ if self.training:
558
+ attn_output, attn_weights = fused_flex_attention(
559
+ query=query_states,
560
+ key=key_states,
561
+ value=value_states,
562
+ attention_mask=attention_mask,
563
+ enable_gqa=True,
564
+ scale=self.scaling,
565
+ return_lse=True
566
+ )
567
+ attn_weights = attn_weights.to(
568
+ value_states.dtype) if attn_weights is not None else None
569
+ attn_output = rearrange(attn_output, 'b h l d -> b l (h d)')
570
+ else:
571
+ attention_mask = attention_mask.bool() if attention_mask is not None else None
572
+ attn_weights = None
573
+ if torch.all(attention_mask): # decoding
574
+ query_states = query_states.transpose(1, 2)
575
+ key_states = key_states.transpose(1, 2)
576
+ value_states = value_states.transpose(1, 2)
577
+ attn_output = flash_attn_func(
578
+ query_states,
579
+ key_states,
580
+ value_states,
581
+ causal=False,
582
+ softmax_scale=self.scaling
583
+ )
584
+ attn_output = rearrange(attn_output, 'b l h d -> b l (h d)')
585
+ else: # prefilling
586
+ attn_output = F.scaled_dot_product_attention(
587
+ query=query_states,
588
+ key=key_states,
589
+ value=value_states,
590
+ attn_mask=attention_mask,
591
+ is_causal=False,
592
+ scale=self.scaling,
593
+ enable_gqa=True
594
+ )
595
+ attn_output = rearrange(attn_output, 'b h l d -> b l (h d)')
596
+ attn_output = self.o_proj(attn_output)
597
+ return attn_output, attn_weights # , attn_weights
598
+
599
+
600
+ class SDARDecoderLayer(GradientCheckpointingLayer):
601
+ def __init__(self, config: SDARConfig, layer_idx: int):
602
+ super().__init__()
603
+ self.hidden_size = config.hidden_size
604
+ self.self_attn = SDARAttention(config=config, layer_idx=layer_idx)
605
+ self.mlp = SDARMLP(config)
606
+ self.input_layernorm = SDARRMSNorm(
607
+ config.hidden_size, eps=config.rms_norm_eps)
608
+ self.post_attention_layernorm = SDARRMSNorm(
609
+ config.hidden_size, eps=config.rms_norm_eps)
610
+ if (
611
+ config.sliding_window and config._attn_implementation != "flash_attention_2"
612
+ ): # diff with Llama is this warning
613
+ logger.warning_once(
614
+ f"Sliding Window Attention is enabled but not implemented for `{config._attn_implementation}`; "
615
+ "unexpected results may be encountered."
616
+ )
617
+
618
+ def forward(
619
+ self,
620
+ hidden_states: torch.Tensor,
621
+ attention_mask: Optional[torch.Tensor] = None,
622
+ position_ids: Optional[torch.LongTensor] = None,
623
+ past_key_value: Optional[Cache] = None,
624
+ output_attentions: Optional[bool] = False,
625
+ use_cache: Optional[bool] = False,
626
+ store_kv: Optional[bool] = False,
627
+ cache_position: Optional[torch.LongTensor] = None,
628
+ # necessary, but kept here for BC
629
+ position_embeddings: Optional[Tuple[torch.Tensor,
630
+ torch.Tensor]] = None,
631
+ **kwargs: Unpack[FlashAttentionKwargs],
632
+ ) -> Tuple[torch.FloatTensor, Optional[Tuple[torch.FloatTensor, torch.FloatTensor]]]:
633
+ residual = hidden_states
634
+ hidden_states = self.input_layernorm(hidden_states)
635
+
636
+ # Self Attention
637
+ hidden_states, self_attn_weights = self.self_attn(
638
+ hidden_states=hidden_states,
639
+ attention_mask=attention_mask,
640
+ position_ids=position_ids,
641
+ past_key_value=past_key_value,
642
+ output_attentions=output_attentions,
643
+ use_cache=use_cache,
644
+ store_kv=store_kv,
645
+ cache_position=cache_position,
646
+ position_embeddings=position_embeddings,
647
+ **kwargs,
648
+ )
649
+ hidden_states = residual + hidden_states
650
+
651
+ # Fully Connected
652
+ residual = hidden_states
653
+ hidden_states = self.post_attention_layernorm(hidden_states)
654
+ hidden_states = self.mlp(hidden_states)
655
+ hidden_states = residual + hidden_states
656
+
657
+ outputs = (hidden_states,)
658
+ if output_attentions:
659
+ outputs += (self_attn_weights,)
660
+
661
+ return outputs
662
+
663
+
664
+ @auto_docstring
665
+ class SDARPreTrainedModel(PreTrainedModel):
666
+ config_class = SDARConfig
667
+ base_model_prefix = "model"
668
+ supports_gradient_checkpointing = True
669
+ _no_split_modules = ["SDARDecoderLayer"]
670
+ _skip_keys_device_placement = ["past_key_values"]
671
+ _supports_flash_attn_2 = True
672
+ _supports_sdpa = True
673
+ _supports_flex_attn = True
674
+ _supports_cache_class = True
675
+ _supports_quantized_cache = True
676
+ _supports_static_cache = True
677
+ _supports_attention_backend = True
678
+
679
+ def _init_weights(self, module):
680
+ std = self.config.initializer_range
681
+ if isinstance(module, nn.Linear):
682
+ module.weight.data.normal_(mean=0.0, std=std)
683
+ if module.bias is not None:
684
+ module.bias.data.zero_()
685
+ elif isinstance(module, nn.Embedding):
686
+ module.weight.data.normal_(mean=0.0, std=std)
687
+ if module.padding_idx is not None:
688
+ module.weight.data[module.padding_idx].zero_()
689
+ elif isinstance(module, SDARRMSNorm):
690
+ module.weight.data.fill_(1.0)
691
+
692
+
693
+ class SDARRotaryEmbedding(nn.Module):
694
+ def __init__(self, config: SDARConfig, device=None):
695
+ super().__init__()
696
+ # BC: "rope_type" was originally "type"
697
+ if hasattr(config, "rope_scaling") and config.rope_scaling is not None:
698
+ self.rope_type = config.rope_scaling.get(
699
+ "rope_type", config.rope_scaling.get("type"))
700
+ else:
701
+ self.rope_type = "default"
702
+ self.max_seq_len_cached = config.max_position_embeddings
703
+ self.original_max_seq_len = config.max_position_embeddings
704
+
705
+ self.config = config
706
+ self.rope_init_fn = ROPE_INIT_FUNCTIONS[self.rope_type]
707
+
708
+ inv_freq, self.attention_scaling = self.rope_init_fn(
709
+ self.config, device)
710
+ self.register_buffer("inv_freq", inv_freq, persistent=False)
711
+ self.original_inv_freq = self.inv_freq
712
+
713
+ @torch.no_grad()
714
+ # power user: used with advanced RoPE types (e.g. dynamic rope)
715
+ @dynamic_rope_update
716
+ def forward(self, x, position_ids):
717
+ inv_freq_expanded = self.inv_freq[None, :, None].float().expand(
718
+ position_ids.shape[0], -1, 1).to(x.device)
719
+ position_ids_expanded = position_ids[:, None, :].float()
720
+
721
+ device_type = x.device.type if isinstance(
722
+ x.device.type, str) and x.device.type != "mps" else "cpu"
723
+ with torch.autocast(device_type=device_type, enabled=False): # Force float32
724
+ freqs = (inv_freq_expanded.float() @
725
+ position_ids_expanded.float()).transpose(1, 2)
726
+ emb = torch.cat((freqs, freqs), dim=-1)
727
+ cos = emb.cos() * self.attention_scaling
728
+ sin = emb.sin() * self.attention_scaling
729
+
730
+ return cos.to(dtype=x.dtype), sin.to(dtype=x.dtype)
731
+
732
+
733
+ @auto_docstring
734
+ class SDARModel(SDARPreTrainedModel):
735
+ def __init__(self, config: SDARConfig):
736
+ super().__init__(config)
737
+ self.padding_idx = config.pad_token_id
738
+ self.vocab_size = config.vocab_size
739
+
740
+ self.embed_tokens = nn.Embedding(
741
+ config.vocab_size, config.hidden_size, self.padding_idx)
742
+ self.layers = nn.ModuleList(
743
+ [SDARDecoderLayer(config, layer_idx)
744
+ for layer_idx in range(config.num_hidden_layers)]
745
+ )
746
+ self.norm = SDARRMSNorm(config.hidden_size, eps=config.rms_norm_eps)
747
+ self.rotary_emb = SDARRotaryEmbedding(config=config)
748
+ self.gradient_checkpointing = False
749
+
750
+ # Initialize weights and apply final processing
751
+ self.post_init()
752
+
753
+ def get_input_embeddings(self):
754
+ return self.embed_tokens
755
+
756
+ def set_input_embeddings(self, value):
757
+ self.embed_tokens = value
758
+
759
+ @can_return_tuple
760
+ @auto_docstring
761
+ def forward(
762
+ self,
763
+ input_ids: Optional[torch.LongTensor] = None,
764
+ attention_mask: Optional[torch.Tensor] = None,
765
+ position_ids: Optional[torch.LongTensor] = None,
766
+ past_key_values: Optional[Cache] = None,
767
+ inputs_embeds: Optional[torch.FloatTensor] = None,
768
+ use_cache: Optional[bool] = None,
769
+ store_kv: Optional[bool] = None,
770
+ output_attentions: Optional[bool] = None,
771
+ output_hidden_states: Optional[bool] = None,
772
+ cache_position: Optional[torch.LongTensor] = None,
773
+ **flash_attn_kwargs: Unpack[FlashAttentionKwargs],
774
+ ) -> BaseModelOutputWithPast:
775
+ output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
776
+ output_hidden_states = (
777
+ output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
778
+ )
779
+ use_cache = use_cache if use_cache is not None else self.config.use_cache
780
+
781
+ if (input_ids is None) ^ (inputs_embeds is not None):
782
+ raise ValueError(
783
+ "You must specify exactly one of input_ids or inputs_embeds")
784
+
785
+ if self.gradient_checkpointing and self.training and use_cache:
786
+ logger.warning_once(
787
+ "`use_cache=True` is incompatible with gradient checkpointing. Setting `use_cache=False`."
788
+ )
789
+ use_cache = False
790
+
791
+ # TODO (joao): remove this exception in v4.56 -- it exists for users that try to pass a legacy cache
792
+ if not isinstance(past_key_values, (type(None), Cache)):
793
+ raise ValueError(
794
+ "The `past_key_values` should be either a `Cache` object or `None`.")
795
+
796
+ if inputs_embeds is None:
797
+ inputs_embeds = self.embed_tokens(input_ids)
798
+
799
+ if use_cache and past_key_values is None:
800
+ past_key_values = DynamicCache()
801
+
802
+ if cache_position is None:
803
+ past_seen_tokens = past_key_values.get_seq_length(
804
+ ) if past_key_values is not None else 0
805
+ cache_position = torch.arange(
806
+ past_seen_tokens, past_seen_tokens + inputs_embeds.shape[1], device=inputs_embeds.device
807
+ )
808
+
809
+ if position_ids is None:
810
+ position_ids = cache_position.unsqueeze(0)
811
+
812
+ # causal_mask = self._update_causal_mask(
813
+ # attention_mask, inputs_embeds, cache_position, past_key_values, output_attentions
814
+ # )
815
+
816
+ hidden_states = inputs_embeds
817
+
818
+ # create position embeddings to be shared across the decoder layers
819
+ position_embeddings = self.rotary_emb(hidden_states, position_ids)
820
+
821
+ # decoder layers
822
+ all_hidden_states = () if output_hidden_states else None
823
+ all_self_attns = () if output_attentions else None
824
+
825
+ for decoder_layer in self.layers[: self.config.num_hidden_layers]:
826
+ if output_hidden_states:
827
+ all_hidden_states += (hidden_states,)
828
+
829
+ layer_outputs = decoder_layer(
830
+ hidden_states,
831
+ attention_mask=attention_mask,
832
+ position_ids=position_ids,
833
+ past_key_value=past_key_values,
834
+ output_attentions=output_attentions,
835
+ use_cache=use_cache,
836
+ store_kv=store_kv,
837
+ cache_position=cache_position,
838
+ position_embeddings=position_embeddings,
839
+ **flash_attn_kwargs,
840
+ )
841
+
842
+ hidden_states = layer_outputs[0]
843
+
844
+ if output_attentions:
845
+ all_self_attns += (layer_outputs[1],)
846
+
847
+ hidden_states = self.norm(hidden_states)
848
+
849
+ # add hidden states from the last decoder layer
850
+ if output_hidden_states:
851
+ all_hidden_states += (hidden_states,)
852
+
853
+ return BaseModelOutputWithPast(
854
+ last_hidden_state=hidden_states,
855
+ past_key_values=past_key_values if use_cache else None,
856
+ hidden_states=all_hidden_states,
857
+ attentions=all_self_attns,
858
+ )
859
+
860
+ def _update_causal_mask(
861
+ self,
862
+ attention_mask: Union[torch.Tensor, "BlockMask"],
863
+ input_tensor: torch.Tensor,
864
+ cache_position: torch.Tensor,
865
+ past_key_values: Cache,
866
+ output_attentions: bool = False,
867
+ ):
868
+ if self.config._attn_implementation == "flash_attention_2":
869
+ if attention_mask is not None and past_key_values is not None:
870
+ is_padding_right = attention_mask[:, -
871
+ 1].sum().item() != input_tensor.size()[0]
872
+ if is_padding_right:
873
+ raise ValueError(
874
+ "You are attempting to perform batched generation with padding_side='right'"
875
+ " this may lead to unexpected behaviour for Flash Attention version of Qwen3. Make sure to "
876
+ " call `tokenizer.padding_side = 'left'` before tokenizing the input. "
877
+ )
878
+ if attention_mask is not None and 0.0 in attention_mask:
879
+ return attention_mask
880
+ return None
881
+ if self.config._attn_implementation == "flex_attention":
882
+ if isinstance(attention_mask, torch.Tensor):
883
+ seq_len_q, seq_len_kv = attention_mask.shape
884
+ assert seq_len_q == seq_len_kv, f"got {attention_mask.shape=}"
885
+ attention_mask = create_block_mask(
886
+ # 2d bool tensor, shape: [2*seqlen, 2*seqlen]
887
+ lambda b, h, q_idx, kv_idx: attention_mask[q_idx, kv_idx],
888
+ B=None, H=None, Q_LEN=seq_len_q, KV_LEN=seq_len_kv,
889
+ )
890
+ else:
891
+ # Here we pass in flex mask computed externally
892
+ assert isinstance(attention_mask, BlockMask)
893
+ return attention_mask
894
+
895
+ # For SDPA, when possible, we will rely on its `is_causal` argument instead of its `attn_mask` argument, in
896
+ # order to dispatch on Flash Attention 2. This feature is not compatible with static cache, as SDPA will fail
897
+ # to infer the attention mask.
898
+ past_seen_tokens = past_key_values.get_seq_length(
899
+ ) if past_key_values is not None else 0
900
+ using_static_cache = isinstance(past_key_values, StaticCache)
901
+ using_sliding_window_cache = isinstance(
902
+ past_key_values, SlidingWindowCache)
903
+
904
+ # When output attentions is True, sdpa implementation's forward method calls the eager implementation's forward
905
+ if (
906
+ self.config._attn_implementation == "sdpa"
907
+ and not (using_static_cache or using_sliding_window_cache)
908
+ and not output_attentions
909
+ ):
910
+ if AttentionMaskConverter._ignore_causal_mask_sdpa(
911
+ attention_mask,
912
+ inputs_embeds=input_tensor,
913
+ past_key_values_length=past_seen_tokens,
914
+ sliding_window=self.config.sliding_window,
915
+ is_training=self.training,
916
+ ):
917
+ return None
918
+
919
+ dtype = input_tensor.dtype
920
+ min_dtype = torch.finfo(dtype).min
921
+ sequence_length = input_tensor.shape[1]
922
+ # SlidingWindowCache or StaticCache
923
+ if using_sliding_window_cache or using_static_cache:
924
+ target_length = past_key_values.get_max_cache_shape()
925
+ # DynamicCache or no cache
926
+ else:
927
+ target_length = (
928
+ attention_mask.shape[-1]
929
+ if isinstance(attention_mask, torch.Tensor)
930
+ else past_seen_tokens + sequence_length + 1
931
+ )
932
+
933
+ # In case the provided `attention` mask is 2D, we generate a causal mask here (4D).
934
+ causal_mask = self._prepare_4d_causal_attention_mask_with_cache_position(
935
+ attention_mask,
936
+ sequence_length=sequence_length,
937
+ target_length=target_length,
938
+ dtype=dtype,
939
+ cache_position=cache_position,
940
+ batch_size=input_tensor.shape[0],
941
+ config=self.config,
942
+ past_key_values=past_key_values,
943
+ )
944
+
945
+ if (
946
+ self.config._attn_implementation == "sdpa"
947
+ and attention_mask is not None
948
+ and attention_mask.device.type in ["cuda", "xpu", "npu"]
949
+ and not output_attentions
950
+ ):
951
+ # Attend to all tokens in fully masked rows in the causal_mask, for example the relevant first rows when
952
+ # using left padding. This is required by F.scaled_dot_product_attention memory-efficient attention path.
953
+ # Details: https://github.com/pytorch/pytorch/issues/110213
954
+ causal_mask = AttentionMaskConverter._unmask_unattended(
955
+ causal_mask, min_dtype)
956
+
957
+ return causal_mask
958
+
959
+ @staticmethod
960
+ def _prepare_4d_causal_attention_mask_with_cache_position(
961
+ attention_mask: torch.Tensor,
962
+ sequence_length: int,
963
+ target_length: int,
964
+ dtype: torch.dtype,
965
+ cache_position: torch.Tensor,
966
+ batch_size: int,
967
+ config: SDARConfig,
968
+ past_key_values: Cache,
969
+ ):
970
+ """
971
+ Creates a causal 4D mask of shape `(batch_size, 1, query_length, key_value_length)` from a 2D mask of shape
972
+ `(batch_size, key_value_length)`, or if the input `attention_mask` is already 4D, do nothing.
973
+
974
+ Args:
975
+ attention_mask (`torch.Tensor`):
976
+ A 2D attention mask of shape `(batch_size, key_value_length)` or a 4D attention mask of shape `(batch_size, 1, query_length, key_value_length)`.
977
+ sequence_length (`int`):
978
+ The sequence length being processed.
979
+ target_length (`int`):
980
+ The target length: when generating with static cache, the mask should be as long as the static cache, to account for the 0 padding, the part of the cache that is not filled yet.
981
+ dtype (`torch.dtype`):
982
+ The dtype to use for the 4D attention mask.
983
+ cache_position (`torch.Tensor`):
984
+ Indices depicting the position of the input sequence tokens in the sequence.
985
+ batch_size (`torch.Tensor`):
986
+ Batch size.
987
+ config (`SDARConfig`):
988
+ The model's configuration class
989
+ past_key_values (`Cache`):
990
+ The cache class that is being used currently to generate
991
+ """
992
+ if attention_mask is not None and attention_mask.dim() == 4:
993
+ # In this case we assume that the mask comes already in inverted form and requires no inversion or slicing.
994
+ causal_mask = attention_mask
995
+ else:
996
+ min_dtype = torch.finfo(dtype).min
997
+ causal_mask = torch.full(
998
+ (sequence_length, target_length), fill_value=min_dtype, dtype=dtype, device=cache_position.device
999
+ )
1000
+ diagonal_attend_mask = torch.arange(target_length, device=cache_position.device) > cache_position.reshape(
1001
+ -1, 1
1002
+ )
1003
+ text_config = config.get_text_config()
1004
+ if getattr(text_config, "use_sliding_window", True) and text_config.sliding_window is not None:
1005
+ # if we have sliding window, we should not attend to tokens beyond sliding window length, so we mask them out also
1006
+ # the check is needed to verify is current checkpoint was trained with sliding window or not
1007
+ if not isinstance(past_key_values, SlidingWindowCache) or sequence_length > target_length:
1008
+ sliding_attend_mask = torch.arange(target_length, device=cache_position.device) <= (
1009
+ cache_position.reshape(-1, 1) -
1010
+ text_config.sliding_window
1011
+ )
1012
+ diagonal_attend_mask.bitwise_or_(sliding_attend_mask)
1013
+ causal_mask *= diagonal_attend_mask
1014
+ causal_mask = causal_mask[None, None,
1015
+ :, :].expand(batch_size, 1, -1, -1)
1016
+ if attention_mask is not None:
1017
+ causal_mask = causal_mask.clone() # copy to contiguous memory for in-place edit
1018
+ if attention_mask.shape[-1] > target_length:
1019
+ attention_mask = attention_mask[:, :target_length]
1020
+ mask_length = attention_mask.shape[-1]
1021
+ padding_mask = causal_mask[:, :, :, :mask_length] + attention_mask[:, None, None, :].to(
1022
+ causal_mask.device
1023
+ )
1024
+ padding_mask = padding_mask == 0
1025
+ causal_mask[:, :, :, :mask_length] = causal_mask[:, :, :, :mask_length].masked_fill(
1026
+ padding_mask, min_dtype
1027
+ )
1028
+ return causal_mask
1029
+
1030
+
1031
+ class KwargsForCausalLM(FlashAttentionKwargs, LossKwargs):
1032
+ ...
1033
+
1034
+
1035
+ @auto_docstring
1036
+ class SDARForCausalLM(SDARPreTrainedModel, GenerationMixin):
1037
+ _tied_weights_keys = ["lm_head.weight"]
1038
+ _tp_plan = {"lm_head": "colwise_rep"}
1039
+ _pp_plan = {"lm_head": (["hidden_states"], ["logits"])}
1040
+
1041
+ def __init__(self, config):
1042
+ super().__init__(config)
1043
+ self.model = SDARModel(config)
1044
+ self.vocab_size = config.vocab_size
1045
+ self.lm_head = nn.Linear(
1046
+ config.hidden_size, config.vocab_size, bias=False)
1047
+
1048
+ # Initialize weights and apply final processing
1049
+ self.post_init()
1050
+
1051
+ def get_input_embeddings(self):
1052
+ return self.model.embed_tokens
1053
+
1054
+ def set_input_embeddings(self, value):
1055
+ self.model.embed_tokens = value
1056
+
1057
+ def get_output_embeddings(self):
1058
+ return self.lm_head
1059
+
1060
+ def set_output_embeddings(self, new_embeddings):
1061
+ self.lm_head = new_embeddings
1062
+
1063
+ def set_decoder(self, decoder):
1064
+ self.model = decoder
1065
+
1066
+ def get_decoder(self):
1067
+ return self.model
1068
+
1069
+ def prepare_for_bd_training(self, inputs_ids, position_ids, prompt_mask):
1070
+ bsz, seq_len = inputs_ids.shape
1071
+ num_tokens = calculate_token_nums(position_ids) # List[torch.Tensor]
1072
+ noisy_inputs_ids, logits_to_keep_half, p_mask = forward_add_noise_packed(
1073
+ inputs_ids=inputs_ids,
1074
+ num_tokens_list=num_tokens,
1075
+ prompt_mask=prompt_mask,
1076
+ mask_id=self.config.mask_token_id,
1077
+ )
1078
+ router_noisy_part_list = []
1079
+ for i in range(bsz):
1080
+ cur_router_noisy_part = (torch.arange(num_tokens[i].shape[0] *2) % 2 == 0).to(inputs_ids.device)
1081
+ cur_router_noisy_part = cur_router_noisy_part.repeat_interleave(num_tokens[i].repeat_interleave(2))
1082
+ router_noisy_part_list.append(cur_router_noisy_part)
1083
+ router_noisy_part = torch.stack(router_noisy_part_list, dim=0)
1084
+
1085
+ # concated inputs_ids: (bzs, seq_len x 2)
1086
+ concat_inputs_ids = inputs_ids.repeat(1, 2)
1087
+ # concated logits_to_keep: (bsz, seq_len x 2)
1088
+ logits_to_keep = torch.zeros(
1089
+ bsz, 2 * seq_len, dtype=torch.bool, device=inputs_ids.device)
1090
+ # concated position_ids: (bsz, seq_len x 2)
1091
+ concat_position_ids = torch.zeros(
1092
+ bsz, 2 * seq_len, dtype=position_ids.dtype, device=position_ids.device)
1093
+ for i in range(bsz):
1094
+ concat_inputs_ids[i][router_noisy_part[i]] = noisy_inputs_ids[i]
1095
+ concat_inputs_ids[i][~router_noisy_part[i]] = inputs_ids[i]
1096
+
1097
+ logits_to_keep[i][router_noisy_part[i]] = logits_to_keep_half[i]
1098
+
1099
+ concat_position_ids[i][router_noisy_part[i]] = position_ids[i]
1100
+ concat_position_ids[i][~router_noisy_part[i]] = position_ids[i]
1101
+
1102
+ # create flex_attention mask
1103
+ attention_mask = block_attn_mask(num_tokens, self.config.block_size, inputs_ids.device)
1104
+ flex_attention_mask_3d = create_block_mask(
1105
+ lambda b, h, q_idx, kv_idx: attention_mask[b, q_idx, kv_idx],
1106
+ B=attention_mask.size(0), H=None,
1107
+ Q_LEN=attention_mask.size(1), KV_LEN=attention_mask.size(2),
1108
+ )
1109
+
1110
+ return concat_inputs_ids, concat_position_ids, flex_attention_mask_3d, logits_to_keep_half, logits_to_keep, p_mask
1111
+
1112
+ @can_return_tuple
1113
+ @auto_docstring
1114
+ def forward(
1115
+ self,
1116
+ input_ids: Optional[torch.LongTensor] = None,
1117
+ attention_mask: Optional[torch.Tensor] = None,
1118
+ position_ids: Optional[torch.LongTensor] = None,
1119
+ past_key_values: Optional[Cache] = None,
1120
+ inputs_embeds: Optional[torch.FloatTensor] = None,
1121
+ labels: Optional[torch.LongTensor] = None,
1122
+ use_cache: Optional[bool] = None,
1123
+ output_attentions: Optional[bool] = None,
1124
+ output_hidden_states: Optional[bool] = None,
1125
+ cache_position: Optional[torch.LongTensor] = None,
1126
+ logits_to_keep: Union[int, torch.Tensor] = 0,
1127
+ **kwargs: Unpack[KwargsForCausalLM],
1128
+ ) -> CausalLMOutputWithPast:
1129
+ r"""
1130
+ labels (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
1131
+ Labels for computing the masked language modeling loss. Indices should either be in `[0, ...,
1132
+ config.vocab_size]` or -100 (see `input_ids` docstring). Tokens with indices set to `-100` are ignored
1133
+ (masked), the loss is only computed for the tokens with labels in `[0, ..., config.vocab_size]`.
1134
+
1135
+ Example:
1136
+
1137
+ ```python
1138
+ >>> from transformers import AutoTokenizer, SDARForCausalLM
1139
+
1140
+ >>> model = SDARForCausalLM.from_pretrained("DiffuOpen/SDAR-1.7B-Chat")
1141
+ >>> tokenizer = AutoTokenizer.from_pretrained("DiffuOpen/SDAR-1.7B-Chat")
1142
+
1143
+ >>> prompt = "Hey, are you conscious? Can you talk to me?"
1144
+ >>> inputs = tokenizer(prompt, return_tensors="pt")
1145
+
1146
+ >>> # Generate
1147
+ >>> generate_ids = model.generate(inputs.input_ids, max_length=30)
1148
+ >>> tokenizer.batch_decode(generate_ids, skip_special_tokens=True, clean_up_tokenization_spaces=False)[0]
1149
+ "Hey, are you conscious? Can you talk to me?\nI'm not conscious, but I can talk to you."
1150
+ ```"""
1151
+ output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
1152
+ output_hidden_states = (
1153
+ output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
1154
+ )
1155
+ if self.training:
1156
+ assert inputs_embeds is None, "only support input_ids during training"
1157
+ prompt_mask = (labels == -100) if labels is not None else None
1158
+ position_ids = modify_padded_position_ids_2d(position_ids)
1159
+ concat_inputs_ids, concat_position_ids, flex_attention_mask_3d, logits_to_keep_half, logits_to_keep, p_mask = self.prepare_for_bd_training(input_ids, position_ids, prompt_mask)
1160
+ outputs = self.model(
1161
+ input_ids=concat_inputs_ids,
1162
+ attention_mask=flex_attention_mask_3d,
1163
+ position_ids=concat_position_ids,
1164
+ output_attentions=output_attentions,
1165
+ output_hidden_states=output_hidden_states,
1166
+ return_dict=True,
1167
+ cache_position=cache_position,
1168
+ **kwargs,
1169
+ )
1170
+ hidden_states = outputs.last_hidden_state
1171
+ hidden_states = hidden_states[logits_to_keep].contiguous()
1172
+ assert labels is not None, "Labels must be provided for training."
1173
+ answer_len = (labels != -100).sum()
1174
+ loss_fct = FusedLinearDiffusionCrossEntropyLoss(reduction='sum')
1175
+ loss = loss_fct( # it will return (sum_loss, unreduced_loss)
1176
+ # conduct `view(-1, V)` inside the function
1177
+ x=hidden_states,
1178
+ target=labels[logits_to_keep_half].contiguous(),
1179
+ weight=self.lm_head.weight,
1180
+ bias=self.lm_head.bias,
1181
+ p_mask=p_mask,
1182
+ )
1183
+ loss = loss / answer_len
1184
+ logits = None
1185
+ else:
1186
+ # decoder outputs consists of (dec_features, layer_state, dec_hidden, dec_attn)
1187
+ outputs: BaseModelOutputWithPast = self.model(
1188
+ input_ids=input_ids,
1189
+ attention_mask=attention_mask,
1190
+ position_ids=position_ids,
1191
+ past_key_values=past_key_values,
1192
+ inputs_embeds=inputs_embeds,
1193
+ use_cache=use_cache,
1194
+ output_attentions=output_attentions,
1195
+ output_hidden_states=output_hidden_states,
1196
+ cache_position=cache_position,
1197
+ **kwargs,
1198
+ )
1199
+
1200
+ hidden_states = outputs.last_hidden_state
1201
+ # Only compute necessary logits, and do not upcast them to float if we are not computing the loss
1202
+ slice_indices = slice(-logits_to_keep,
1203
+ None) if isinstance(logits_to_keep, int) else logits_to_keep
1204
+ hidden_states = hidden_states[:, slice_indices, :].contiguous()
1205
+ fuse_linear_and_cross_entropy = self.config.fuse_cross_entropy and self.training
1206
+ if fuse_linear_and_cross_entropy:
1207
+ # When using fused_linear_ce_loss, we do not compute the whole logits on HBM
1208
+ logits = None
1209
+ else:
1210
+ logits = self.lm_head(hidden_states)
1211
+
1212
+ loss = None
1213
+ if labels is not None:
1214
+ # FusedLinearCrossEntropyLoss will be implemented by monkey patch when training
1215
+ # We don't use it when inferencing
1216
+ loss_fct = nn.CrossEntropyLoss() # nn.CE
1217
+ loss = loss_fct(
1218
+ logits.view(-1, self.config.vocab_size), labels.view(-1))
1219
+
1220
+ return CausalLMOutputWithPast(
1221
+ loss=loss,
1222
+ logits=logits,
1223
+ past_key_values=outputs.past_key_values,
1224
+ hidden_states=outputs.hidden_states,
1225
+ attentions=outputs.attentions,
1226
+ )
1227
+
1228
+
1229
+ __all__ = [
1230
+ "SDARForCausalLM",
1231
+ "SDARModel",
1232
+ "SDARPreTrainedModel",
1233
+ ]
Qwen3.4B-Math-R1-CoT-SFT/special_tokens_map.json CHANGED
@@ -12,7 +12,8 @@
12
  "<|vision_end|>",
13
  "<|vision_pad|>",
14
  "<|image_pad|>",
15
- "<|video_pad|>"
 
16
  ],
17
  "eos_token": {
18
  "content": "<|im_end|>",
@@ -21,6 +22,13 @@
21
  "rstrip": false,
22
  "single_word": false
23
  },
 
 
 
 
 
 
 
24
  "pad_token": {
25
  "content": "<|endoftext|>",
26
  "lstrip": false,
 
12
  "<|vision_end|>",
13
  "<|vision_pad|>",
14
  "<|image_pad|>",
15
+ "<|video_pad|>",
16
+ "<|MASK|>"
17
  ],
18
  "eos_token": {
19
  "content": "<|im_end|>",
 
22
  "rstrip": false,
23
  "single_word": false
24
  },
25
+ "mask_token": {
26
+ "content": "<|MASK|>",
27
+ "lstrip": false,
28
+ "normalized": false,
29
+ "rstrip": false,
30
+ "single_word": false
31
+ },
32
  "pad_token": {
33
  "content": "<|endoftext|>",
34
  "lstrip": false,
Qwen3.4B-Math-R1-CoT-SFT/tokenization_qwen2.py ADDED
@@ -0,0 +1,342 @@
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
1
+ # coding=utf-8
2
+ # Copyright 2024 The Qwen team, Alibaba Group and The HuggingFace Inc. team. All rights reserved.
3
+ #
4
+ # Licensed under the Apache License, Version 2.0 (the "License");
5
+ # you may not use this file except in compliance with the License.
6
+ # You may obtain a copy of the License at
7
+ #
8
+ # http://www.apache.org/licenses/LICENSE-2.0
9
+ #
10
+ # Unless required by applicable law or agreed to in writing, software
11
+ # distributed under the License is distributed on an "AS IS" BASIS,
12
+ # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
13
+ # See the License for the specific language governing permissions and
14
+ # limitations under the License.
15
+ """Tokenization classes for Qwen2."""
16
+
17
+ import json
18
+ import os
19
+ import unicodedata
20
+ from functools import lru_cache
21
+ from typing import Optional, Tuple
22
+
23
+ import regex as re
24
+
25
+ from transformers.tokenization_utils import AddedToken, PreTrainedTokenizer
26
+ from transformers.utils import logging
27
+
28
+
29
+ logger = logging.get_logger(__name__)
30
+
31
+ VOCAB_FILES_NAMES = {
32
+ "vocab_file": "vocab.json",
33
+ "merges_file": "merges.txt",
34
+ }
35
+
36
+
37
+ MAX_MODEL_INPUT_SIZES = {"qwen/qwen-tokenizer": 32768}
38
+
39
+ PRETOKENIZE_REGEX = r"""(?i:'s|'t|'re|'ve|'m|'ll|'d)|[^\r\n\p{L}\p{N}]?\p{L}+|\p{N}| ?[^\s\p{L}\p{N}]+[\r\n]*|\s*[\r\n]+|\s+(?!\S)|\s+"""
40
+
41
+
42
+ @lru_cache()
43
+ # Copied from transformers.models.gpt2.tokenization_gpt2.bytes_to_unicode
44
+ def bytes_to_unicode():
45
+ """
46
+ Returns list of utf-8 byte and a mapping to unicode strings. We specifically avoids mapping to whitespace/control
47
+ characters the bpe code barfs on.
48
+
49
+ The reversible bpe codes work on unicode strings. This means you need a large # of unicode characters in your vocab
50
+ if you want to avoid UNKs. When you're at something like a 10B token dataset you end up needing around 5K for
51
+ decent coverage. This is a significant percentage of your normal, say, 32K bpe vocab. To avoid that, we want lookup
52
+ tables between utf-8 bytes and unicode strings.
53
+ """
54
+ bs = (
55
+ list(range(ord("!"), ord("~") + 1)) + list(range(ord("¡"), ord("¬") + 1)) + list(range(ord("®"), ord("ÿ") + 1))
56
+ )
57
+ cs = bs[:]
58
+ n = 0
59
+ for b in range(2**8):
60
+ if b not in bs:
61
+ bs.append(b)
62
+ cs.append(2**8 + n)
63
+ n += 1
64
+ cs = [chr(n) for n in cs]
65
+ return dict(zip(bs, cs))
66
+
67
+
68
+ # Copied from transformers.models.gpt2.tokenization_gpt2.get_pairs
69
+ def get_pairs(word):
70
+ """
71
+ Return set of symbol pairs in a word.
72
+
73
+ Word is represented as tuple of symbols (symbols being variable-length strings).
74
+ """
75
+ pairs = set()
76
+ prev_char = word[0]
77
+ for char in word[1:]:
78
+ pairs.add((prev_char, char))
79
+ prev_char = char
80
+ return pairs
81
+
82
+
83
+ class Qwen2Tokenizer(PreTrainedTokenizer):
84
+ """
85
+ Construct a Qwen2 tokenizer. Based on byte-level Byte-Pair-Encoding.
86
+
87
+ Same with GPT2Tokenizer, this tokenizer has been trained to treat spaces like parts of the tokens so a word will
88
+ be encoded differently whether it is at the beginning of the sentence (without space) or not:
89
+
90
+ ```python
91
+ >>> from transformers import Qwen2Tokenizer
92
+
93
+ >>> tokenizer = Qwen2Tokenizer.from_pretrained("Qwen/Qwen-tokenizer")
94
+ >>> tokenizer("Hello world")["input_ids"]
95
+ [9707, 1879]
96
+
97
+ >>> tokenizer(" Hello world")["input_ids"]
98
+ [21927, 1879]
99
+ ```
100
+ This is expected.
101
+
102
+ You should not use GPT2Tokenizer instead, because of the different pretokenization rules.
103
+
104
+ This tokenizer inherits from [`PreTrainedTokenizer`] which contains most of the main methods. Users should refer to
105
+ this superclass for more information regarding those methods.
106
+
107
+ Args:
108
+ vocab_file (`str`):
109
+ Path to the vocabulary file.
110
+ merges_file (`str`):
111
+ Path to the merges file.
112
+ errors (`str`, *optional*, defaults to `"replace"`):
113
+ Paradigm to follow when decoding bytes to UTF-8. See
114
+ [bytes.decode](https://docs.python.org/3/library/stdtypes.html#bytes.decode) for more information.
115
+ unk_token (`str`, *optional*, defaults to `"<|endoftext|>"`):
116
+ The unknown token. A token that is not in the vocabulary cannot be converted to an ID and is set to be this
117
+ token instead.
118
+ bos_token (`str`, *optional*):
119
+ The beginning of sequence token. Not applicable for this tokenizer.
120
+ eos_token (`str`, *optional*, defaults to `"<|endoftext|>"`):
121
+ The end of sequence token.
122
+ pad_token (`str`, *optional*, defaults to `"<|endoftext|>"`):
123
+ The token used for padding, for example when batching sequences of different lengths.
124
+ clean_up_tokenization_spaces (`bool`, *optional*, defaults to `False`):
125
+ Whether or not the model should cleanup the spaces that were added when splitting the input text during the
126
+ tokenization process. Not applicable to this tokenizer, since tokenization does not add spaces.
127
+ split_special_tokens (`bool`, *optional*, defaults to `False`):
128
+ Whether or not the special tokens should be split during the tokenization process. The default behavior is
129
+ to not split special tokens. This means that if `<|endoftext|>` is the `eos_token`, then `tokenizer.tokenize("<|endoftext|>") =
130
+ ['<|endoftext|>`]. Otherwise, if `split_special_tokens=True`, then `tokenizer.tokenize("<|endoftext|>")` will be give `['<',
131
+ '|', 'endo', 'ft', 'ext', '|', '>']`. This argument is only supported for `slow` tokenizers for the moment.
132
+ """
133
+
134
+ vocab_files_names = VOCAB_FILES_NAMES
135
+ model_input_names = ["input_ids", "attention_mask"]
136
+
137
+ def __init__(
138
+ self,
139
+ vocab_file,
140
+ merges_file,
141
+ errors="replace",
142
+ unk_token="<|endoftext|>",
143
+ bos_token=None,
144
+ eos_token="<|endoftext|>",
145
+ pad_token="<|endoftext|>",
146
+ clean_up_tokenization_spaces=False,
147
+ split_special_tokens=False,
148
+ **kwargs,
149
+ ):
150
+ # Qwen vocab does not contain control tokens; added tokens need to be special
151
+ bos_token = (
152
+ AddedToken(bos_token, lstrip=False, rstrip=False, special=True, normalized=False)
153
+ if isinstance(bos_token, str)
154
+ else bos_token
155
+ )
156
+ eos_token = (
157
+ AddedToken(eos_token, lstrip=False, rstrip=False, special=True, normalized=False)
158
+ if isinstance(eos_token, str)
159
+ else eos_token
160
+ )
161
+ unk_token = (
162
+ AddedToken(unk_token, lstrip=False, rstrip=False, special=True, normalized=False)
163
+ if isinstance(unk_token, str)
164
+ else unk_token
165
+ )
166
+ pad_token = (
167
+ AddedToken(pad_token, lstrip=False, rstrip=False, special=True, normalized=False)
168
+ if isinstance(pad_token, str)
169
+ else pad_token
170
+ )
171
+
172
+ with open(vocab_file, encoding="utf-8") as vocab_handle:
173
+ self.encoder = json.load(vocab_handle)
174
+ self.decoder = {v: k for k, v in self.encoder.items()}
175
+ self.errors = errors # how to handle errors in decoding
176
+ self.byte_encoder = bytes_to_unicode()
177
+ self.byte_decoder = {v: k for k, v in self.byte_encoder.items()}
178
+ bpe_merges = []
179
+ with open(merges_file, encoding="utf-8") as merges_handle:
180
+ for i, line in enumerate(merges_handle):
181
+ line = line.strip()
182
+ if (i == 0 and line.startswith("#version:")) or not line:
183
+ continue
184
+ bpe_merges.append(tuple(line.split()))
185
+ self.bpe_ranks = dict(zip(bpe_merges, range(len(bpe_merges))))
186
+ # NOTE: the cache can grow without bound and will get really large for long running processes
187
+ # (esp. for texts of language that do not use space between word, e.g. Chinese); technically
188
+ # not a memory leak but appears as one.
189
+ # GPT2Tokenizer has the same problem, so let's be consistent.
190
+ self.cache = {}
191
+
192
+ self.pat = re.compile(PRETOKENIZE_REGEX)
193
+
194
+ if kwargs.get("add_prefix_space", False):
195
+ logger.warning_once(
196
+ f"{self.__class__.__name} does not support `add_prefix_space`, setting it to True has no effect."
197
+ )
198
+
199
+ super().__init__(
200
+ errors=errors,
201
+ bos_token=bos_token,
202
+ eos_token=eos_token,
203
+ pad_token=pad_token,
204
+ unk_token=unk_token,
205
+ clean_up_tokenization_spaces=clean_up_tokenization_spaces,
206
+ split_special_tokens=split_special_tokens,
207
+ **kwargs,
208
+ )
209
+
210
+ @property
211
+ def vocab_size(self) -> int:
212
+ return len(self.encoder)
213
+
214
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer.get_vocab
215
+ def get_vocab(self):
216
+ return dict(self.encoder, **self.added_tokens_encoder)
217
+
218
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer.bpe
219
+ def bpe(self, token):
220
+ if token in self.cache:
221
+ return self.cache[token]
222
+ word = tuple(token)
223
+ pairs = get_pairs(word)
224
+
225
+ if not pairs:
226
+ return token
227
+
228
+ while True:
229
+ bigram = min(pairs, key=lambda pair: self.bpe_ranks.get(pair, float("inf")))
230
+ if bigram not in self.bpe_ranks:
231
+ break
232
+ first, second = bigram
233
+ new_word = []
234
+ i = 0
235
+ while i < len(word):
236
+ try:
237
+ j = word.index(first, i)
238
+ except ValueError:
239
+ new_word.extend(word[i:])
240
+ break
241
+ else:
242
+ new_word.extend(word[i:j])
243
+ i = j
244
+
245
+ if word[i] == first and i < len(word) - 1 and word[i + 1] == second:
246
+ new_word.append(first + second)
247
+ i += 2
248
+ else:
249
+ new_word.append(word[i])
250
+ i += 1
251
+ new_word = tuple(new_word)
252
+ word = new_word
253
+ if len(word) == 1:
254
+ break
255
+ else:
256
+ pairs = get_pairs(word)
257
+ word = " ".join(word)
258
+ self.cache[token] = word
259
+ return word
260
+
261
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer._tokenize
262
+ def _tokenize(self, text):
263
+ """Tokenize a string."""
264
+ bpe_tokens = []
265
+ for token in re.findall(self.pat, text):
266
+ token = "".join(
267
+ self.byte_encoder[b] for b in token.encode("utf-8")
268
+ ) # Maps all our bytes to unicode strings, avoiding control tokens of the BPE (spaces in our case)
269
+ bpe_tokens.extend(bpe_token for bpe_token in self.bpe(token).split(" "))
270
+ return bpe_tokens
271
+
272
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer._convert_token_to_id
273
+ def _convert_token_to_id(self, token):
274
+ """Converts a token (str) in an id using the vocab."""
275
+ return self.encoder.get(token, self.encoder.get(self.unk_token))
276
+
277
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer._convert_id_to_token
278
+ def _convert_id_to_token(self, index):
279
+ """Converts an index (integer) in a token (str) using the vocab."""
280
+ return self.decoder.get(index)
281
+
282
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer.convert_tokens_to_string
283
+ def convert_tokens_to_string(self, tokens):
284
+ """Converts a sequence of tokens (string) in a single string."""
285
+ text = "".join(tokens)
286
+ text = bytearray([self.byte_decoder[c] for c in text]).decode("utf-8", errors=self.errors)
287
+ return text
288
+
289
+ def decode(
290
+ self,
291
+ token_ids,
292
+ skip_special_tokens: bool = False,
293
+ clean_up_tokenization_spaces: Optional[bool] = False,
294
+ spaces_between_special_tokens: bool = False,
295
+ **kwargs,
296
+ ) -> str:
297
+ # `spaces_between_special_tokens` defaults to True for _decode in slow tokenizers
298
+ # and cannot be configured elsewhere, but it should default to False for Qwen2Tokenizer
299
+ return super().decode(
300
+ token_ids,
301
+ skip_special_tokens=skip_special_tokens,
302
+ clean_up_tokenization_spaces=clean_up_tokenization_spaces,
303
+ spaces_between_special_tokens=spaces_between_special_tokens,
304
+ **kwargs,
305
+ )
306
+
307
+ # Copied from transformers.models.gpt2.tokenization_gpt2.GPT2Tokenizer.save_vocabulary
308
+ def save_vocabulary(self, save_directory: str, filename_prefix: Optional[str] = None) -> Tuple[str]:
309
+ if not os.path.isdir(save_directory):
310
+ logger.error(f"Vocabulary path ({save_directory}) should be a directory")
311
+ return
312
+ vocab_file = os.path.join(
313
+ save_directory, (filename_prefix + "-" if filename_prefix else "") + VOCAB_FILES_NAMES["vocab_file"]
314
+ )
315
+ merge_file = os.path.join(
316
+ save_directory, (filename_prefix + "-" if filename_prefix else "") + VOCAB_FILES_NAMES["merges_file"]
317
+ )
318
+
319
+ with open(vocab_file, "w", encoding="utf-8") as f:
320
+ f.write(json.dumps(self.encoder, indent=2, sort_keys=True, ensure_ascii=False) + "\n")
321
+
322
+ index = 0
323
+ with open(merge_file, "w", encoding="utf-8") as writer:
324
+ writer.write("#version: 0.2\n")
325
+ for bpe_tokens, token_index in sorted(self.bpe_ranks.items(), key=lambda kv: kv[1]):
326
+ if index != token_index:
327
+ logger.warning(
328
+ f"Saving vocabulary to {merge_file}: BPE merge indices are not consecutive."
329
+ " Please check that the tokenizer is not corrupted!"
330
+ )
331
+ index = token_index
332
+ writer.write(" ".join(bpe_tokens) + "\n")
333
+ index += 1
334
+
335
+ return vocab_file, merge_file
336
+
337
+ def prepare_for_tokenization(self, text, **kwargs):
338
+ text = unicodedata.normalize("NFC", text)
339
+ return (text, kwargs)
340
+
341
+
342
+ __all__ = ["Qwen2Tokenizer"]
Qwen3.4B-Math-R1-CoT-SFT/tokenizer_config.json CHANGED
@@ -209,6 +209,14 @@
209
  "rstrip": false,
210
  "single_word": false,
211
  "special": false
 
 
 
 
 
 
 
 
212
  }
213
  },
214
  "additional_special_tokens": [
@@ -224,13 +232,21 @@
224
  "<|vision_end|>",
225
  "<|vision_pad|>",
226
  "<|image_pad|>",
227
- "<|video_pad|>"
 
228
  ],
 
 
 
 
 
 
229
  "bos_token": null,
230
  "clean_up_tokenization_spaces": false,
231
  "eos_token": "<|im_end|>",
232
  "errors": "replace",
233
  "extra_special_tokens": {},
 
234
  "model_max_length": 131072,
235
  "pad_token": "<|endoftext|>",
236
  "padding_side": "right",
 
209
  "rstrip": false,
210
  "single_word": false,
211
  "special": false
212
+ },
213
+ "151669": {
214
+ "content": "<|MASK|>",
215
+ "lstrip": false,
216
+ "normalized": false,
217
+ "rstrip": false,
218
+ "single_word": false,
219
+ "special": true
220
  }
221
  },
222
  "additional_special_tokens": [
 
232
  "<|vision_end|>",
233
  "<|vision_pad|>",
234
  "<|image_pad|>",
235
+ "<|video_pad|>",
236
+ "<|MASK|>"
237
  ],
238
+ "auto_map": {
239
+ "AutoTokenizer": [
240
+ "tokenization_qwen2.Qwen2Tokenizer",
241
+ null
242
+ ]
243
+ },
244
  "bos_token": null,
245
  "clean_up_tokenization_spaces": false,
246
  "eos_token": "<|im_end|>",
247
  "errors": "replace",
248
  "extra_special_tokens": {},
249
+ "mask_token": "<|MASK|>",
250
  "model_max_length": 131072,
251
  "pad_token": "<|endoftext|>",
252
  "padding_side": "right",
Qwen3.4B-Math-R1-CoT-SFT/train_results.json CHANGED
@@ -1,9 +1,9 @@
1
  {
2
- "effective_tokens_per_sec": 24290.389169769056,
3
- "epoch": 4.0,
4
- "total_flos": 4.3530238414695694e+19,
5
- "train_loss": 0.15123671168837785,
6
- "train_runtime": 82201.3053,
7
- "train_samples_per_second": 1.977,
8
- "train_steps_per_second": 0.031
9
  }
 
1
  {
2
+ "effective_tokens_per_sec": 16342.776477265266,
3
+ "epoch": 3.9988184324537217,
4
+ "total_flos": 4.3565600076592054e+19,
5
+ "train_loss": 1.895302517482642,
6
+ "train_runtime": 61070.1581,
7
+ "train_samples_per_second": 2.661,
8
+ "train_steps_per_second": 0.042
9
  }
Qwen3.4B-Math-R1-CoT-SFT/trainer_log.jsonl CHANGED
The diff for this file is too large to render. See raw diff
 
Qwen3.4B-Math-R1-CoT-SFT/trainer_state.json CHANGED
The diff for this file is too large to render. See raw diff
 
Qwen3.4B-Math-R1-CoT-SFT/training_args.bin CHANGED
@@ -1,3 +1,3 @@
1
  version https://git-lfs.github.com/spec/v1
2
- oid sha256:c08f76ab47b206e4b9c06d41cfab36618b2faaeae2f1c40b815ba9fb75cc2770
3
  size 7825
 
1
  version https://git-lfs.github.com/spec/v1
2
+ oid sha256:46943a038bbd8afe345ee35cc3e2d8972bcd53d59b88da7fd7b677b1c4414324
3
  size 7825
Qwen3.4B-Math-R1-CoT-SFT/training_loss.png CHANGED
Qwen3.4B-Math-R1-CoT-SFT/vocab.json CHANGED
The diff for this file is too large to render. See raw diff